from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, EPMTaskTemplate, ItemRevision, RevisionRule
from tcsoa.gen.Internal.AWS2._2012_10.FullTextSearch import GetTcObjectsToIndexInput, PreFiltersResponse, GetObjectsToIndexResponse, SearchSuggestionsResponse, CreateFullTextSavedSearchResponse, GetImpactedItemRevsForReIndexResponse, UpdateIndexerStatusInput, FullTextSearchResponse, GetDeletedObjectsToUpdateIndexResponse, SearchSuggestionsInput, GetAddedObjectsToUpdateIndexResponse, GetDeletedObjectsToUpdateIndexResponse1, GetDatasetIndexableFilesInfoResponse, FullTextSavedSearchInput, FindFullTextSavedSearchesResponse, GetAddedObjectsToUpdateIndexResponse1, FullTextSearchInput
from tcsoa.gen.Internal.AWS2._2012_10.RequirementsManagement import ExportToApplicationResponse, SetContentInput, ExportToApplicationInputData
from tcsoa.gen.Internal.AWS2._2012_10.Finder import FindObjectsResponse2, FindUsersTasksResponse, FindObjectsInput2
from tcsoa.gen.Internal.AWS2._2012_10.DataManagement import GetChildrenResponse, SaveEditInfo, GetTCSessionInfoResponse, SaveEditAndSubmitInfo, HistoryInput, HistoryResult, GetCurrentUserGatewayResponse, LoadDataForEditingInfo, SaveEditAndSubmitResponse, LoadDataForEditingResponse
from tcsoa.gen.Internal.AWS2._2012_10.Workflow import GetTaskResultsResponse
from tcsoa.gen.Internal.AWS2._2012_10.OrganizationManagement import GroupMembershipResponse, GroupMembershipInput
from tcsoa.gen.Internal.AWS2._2012_10.LOV import LOVData, LOVSearchResults, InitialLovData, LOVInput, ValidateLOVValueSelectionsResponse
from typing import List
from tcsoa.gen.Internal.AWS2._2012_10.StructureSearch import ProductsWhereUsedRespone, ParentsWhereUsedResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def getParentsWhereUsed(cls, itemRevisions: List[ItemRevision], revisionRule: RevisionRule, pageNumber: int) -> ParentsWhereUsedResponse:
        """
        The response of this operation is the list of Parent Item Revisions where each one of the input Item Revisions
        are used.
        """
        return cls.execute_soa_method(
            method_name='getParentsWhereUsed',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='StructureSearch',
            params={'itemRevisions': itemRevisions, 'revisionRule': revisionRule, 'pageNumber': pageNumber},
            response_cls=ParentsWhereUsedResponse,
        )

    @classmethod
    def getProductsWhereUsed(cls, itemRevisions: List[ItemRevision], pageNumber: int) -> ProductsWhereUsedRespone:
        """
        The response of this operation is the list of Product Items where each one of the input Item Revisions are used.
        """
        return cls.execute_soa_method(
            method_name='getProductsWhereUsed',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='StructureSearch',
            params={'itemRevisions': itemRevisions, 'pageNumber': pageNumber},
            response_cls=ProductsWhereUsedRespone,
        )


class FullTextSearchService(TcService):

    @classmethod
    def getPreFilters(cls) -> PreFiltersResponse:
        """
        Fetches the pre filter properties and their values from the preferences defined in the system.
        """
        return cls.execute_soa_method(
            method_name='getPreFilters',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={},
            response_cls=PreFiltersResponse,
        )

    @classmethod
    def getSuggestions(cls, searchInput: SearchSuggestionsInput) -> SearchSuggestionsResponse:
        """
        This operation returns a list of search suggestions for the given search string.
        """
        return cls.execute_soa_method(
            method_name='getSuggestions',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'searchInput': searchInput},
            response_cls=SearchSuggestionsResponse,
        )

    @classmethod
    def performFullTextSearch(cls, searchInput: FullTextSearchInput) -> FullTextSearchResponse:
        """
        This operation returns a list of business objects obtained after performing a full text search using the input
        search criteria.
        """
        return cls.execute_soa_method(
            method_name='performFullTextSearch',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'searchInput': searchInput},
            response_cls=FullTextSearchResponse,
        )

    @classmethod
    def updateIndexingStatus(cls, indexerStatusInput: UpdateIndexerStatusInput) -> ServiceData:
        """
        Updates the subscription table with the application id and processed date time of an action during indexing
        process.
        """
        return cls.execute_soa_method(
            method_name='updateIndexingStatus',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'indexerStatusInput': indexerStatusInput},
            response_cls=ServiceData,
        )

    @classmethod
    def createFullTextSavedSearch(cls, inputs: List[FullTextSavedSearchInput]) -> CreateFullTextSavedSearchResponse:
        """
        Create FullTextSavedSearch objects
        """
        return cls.execute_soa_method(
            method_name='createFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'inputs': inputs},
            response_cls=CreateFullTextSavedSearchResponse,
        )

    @classmethod
    def findFullTextSavedSearches(cls) -> FindFullTextSavedSearchesResponse:
        """
        Find FullTextSavedSearch objects
        """
        return cls.execute_soa_method(
            method_name='findFullTextSavedSearches',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={},
            response_cls=FindFullTextSavedSearchesResponse,
        )

    @classmethod
    def getAddedObjectsToUpdateIndex(cls, applicationID: str) -> GetAddedObjectsToUpdateIndexResponse:
        """
        This operation queries the scratch table to identify objects that have been added since the last indexing
        operation.
        """
        return cls.execute_soa_method(
            method_name='getAddedObjectsToUpdateIndex',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'applicationID': applicationID},
            response_cls=GetAddedObjectsToUpdateIndexResponse,
        )

    @classmethod
    def getAddedObjectsToUpdateIndex1(cls, applicationID: str) -> GetAddedObjectsToUpdateIndexResponse1:
        """
        This operation queries the scratch table to identify objects that have been added since the last indexing
        operation.
        """
        return cls.execute_soa_method(
            method_name='getAddedObjectsToUpdateIndex1',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'applicationID': applicationID},
            response_cls=GetAddedObjectsToUpdateIndexResponse1,
        )

    @classmethod
    def getDatasetIndexableFilesInfo(cls, datasetUIDs: List[str]) -> GetDatasetIndexableFilesInfoResponse:
        """
        This operation finds all the files associated with the given list of datasets and filters out that are not
        supported for indexing. Finally returns back a map of datasets uids and supported files information of file UID
        and FMS read ticket for downloading the file.
        """
        return cls.execute_soa_method(
            method_name='getDatasetIndexableFilesInfo',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'datasetUIDs': datasetUIDs},
            response_cls=GetDatasetIndexableFilesInfoResponse,
        )

    @classmethod
    def getDeletedObjectsToUpdateIndex(cls, applicationID: str) -> GetDeletedObjectsToUpdateIndexResponse:
        """
        This operation queries the scratch table to identify objects that have been deleted since the last indexing
        operation.
        """
        return cls.execute_soa_method(
            method_name='getDeletedObjectsToUpdateIndex',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'applicationID': applicationID},
            response_cls=GetDeletedObjectsToUpdateIndexResponse,
        )

    @classmethod
    def getDeletedObjectsToUpdateIndex1(cls, applicationID: str) -> GetDeletedObjectsToUpdateIndexResponse1:
        """
        This operation queries the scratch table to identify objects that have been deleted since last indexing
        operation.
        """
        return cls.execute_soa_method(
            method_name='getDeletedObjectsToUpdateIndex1',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'applicationID': applicationID},
            response_cls=GetDeletedObjectsToUpdateIndexResponse1,
        )

    @classmethod
    def getImpactedItemRevsForReIndex(cls) -> GetImpactedItemRevsForReIndexResponse:
        """
        The operation queries on scratch, accountability and subscription tables to identify the impacted ItemRevisions
        from last index time due to adds/modify/deletes that took place. These impacted objects need to be re-indexed
        to have the revision rule selectors updated.
        Steps:
        1.    Queries for deleted objects from scratch table. Gets their parent Items from accountability table 
        impacted island if they exist. For the Items found, it fetches all the ItemRevisions and adds them to the
        response.
        2.    Queries for newly added objects from scratch table. Filters out non ItemRevisions from the list. Adds the
        ItemRevisions to the reponse.
        3.    Queries for any Modified islands from the accountability table. If modified island belongs to an
        ItemRevision, then the ItemRevision will be added to the response.
        
        Finally the response will contain the impacted ItemRevisions due to adds/modify/deletes.
        
        
        Use cases:
        Use Case 1: TcFtsIndexer Sync
        
        When data is indexed for the first time the Revision rule slector information is also indexed on each
        ItemRevision. So whenever  a new ItemRevision is added/deleted/modified that impacts all the ItemRevisions
        associated with its parent Item and the revision rule selector information need to be regenerated. This new
        operation will help in identifying the impacted objects and they can be re-indexed to sync the index data .
        """
        return cls.execute_soa_method(
            method_name='getImpactedItemRevsForReIndex',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={},
            response_cls=GetImpactedItemRevsForReIndexResponse,
        )

    @classmethod
    def getObjectsToIndex(cls, objectsToIndexInput: GetTcObjectsToIndexInput) -> GetObjectsToIndexResponse:
        """
        Determine the list of objects to index based on the given date/time range and the "Awp0SearchIsIndexed"
        business object constant.
        """
        return cls.execute_soa_method(
            method_name='getObjectsToIndex',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='FullTextSearch',
            params={'objectsToIndexInput': objectsToIndexInput},
            response_cls=GetObjectsToIndexResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def getTCSessionInfo(cls) -> GetTCSessionInfoResponse:
        """
        This operation gets information about the current user's Teamcenter session.
        """
        return cls.execute_soa_method(
            method_name='getTCSessionInfo',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={},
            response_cls=GetTCSessionInfoResponse,
        )

    @classmethod
    def loadDataForEditing(cls, inputs: List[LoadDataForEditingInfo]) -> LoadDataForEditingResponse:
        """
        This SOA method ensures that the properties can be edited, and returns the last save date of the related
        objects for optimistic edit
        """
        return cls.execute_soa_method(
            method_name='loadDataForEditing',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=LoadDataForEditingResponse,
        )

    @classmethod
    def saveEdit(cls, inputs: List[SaveEditInfo]) -> ServiceData:
        """
        Save the edits on the objects
        """
        return cls.execute_soa_method(
            method_name='saveEdit',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def saveEditAndSubmitToWorkflow(cls, inputs: List[SaveEditAndSubmitInfo]) -> SaveEditAndSubmitResponse:
        """
        This generic operation saves the modified properties for the given input objects and submits them to workflow.
        First it will perform the save operation and if completed successfully, will initiate a workflow process for
        all input objects.
        
        Use cases:
        User can modify the object(s) properties and submit the object(s) to workflow in one operation. This operation
        first saves the modified properties and then initiates the workflow process for all input objects.
        """
        return cls.execute_soa_method(
            method_name='saveEditAndSubmitToWorkflow',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveEditAndSubmitResponse,
        )

    @classmethod
    def updateHistory(cls, historyInput: HistoryInput) -> HistoryResult:
        """
        This operation adds objects to the history and/or removes objects from the history.
        """
        return cls.execute_soa_method(
            method_name='updateHistory',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={'historyInput': historyInput},
            response_cls=HistoryResult,
        )

    @classmethod
    def clearHistory(cls) -> HistoryResult:
        """
        This operation clears the objects in the history and returns the objects that were present in the history.
        """
        return cls.execute_soa_method(
            method_name='clearHistory',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={},
            response_cls=HistoryResult,
        )

    @classmethod
    def getChildren(cls, input: List[BusinessObject]) -> GetChildrenResponse:
        """
        getChildren
        """
        return cls.execute_soa_method(
            method_name='getChildren',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetChildrenResponse,
        )

    @classmethod
    def getCurrentUserGateway(cls) -> GetCurrentUserGatewayResponse:
        """
        The operation looks for the Awp0TileCollection with current logged in user as scope and returns information
        about the Awp0Tile objects associated to it.
        
        Use cases:
        This operation is suited to display the Awp0Tile objects on the Active Workspace client gateway page. The
        Awp0Tiles that are returned by this operation are the ones which are configured for the logged in user based on
        current group, role, and project. 
        
        Display the user specific Awp0Tile objects during startup
        - User logs into Active Workspace.
        - Active Workspace invokes 'getCurrentUserGateway' operation.
        - Displays the Awp0Tile that are configured for the logged in user.
        
        
        
        Display the user specific Awp0Tile objects during context change
        - User changes the group/role/project in Active Workspace.
        - Active Workspace invokes 'getCurrentUserGateway' operation.
        - Displays the Awp0Tile that are configured for the logged in user based on current group, role, and project.
        
        """
        return cls.execute_soa_method(
            method_name='getCurrentUserGateway',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={},
            response_cls=GetCurrentUserGatewayResponse,
        )

    @classmethod
    def getHistory(cls) -> HistoryResult:
        """
        This operation returns the objects in the history.
        """
        return cls.execute_soa_method(
            method_name='getHistory',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='DataManagement',
            params={},
            response_cls=HistoryResult,
        )


class WorkflowService(TcService):

    @classmethod
    def getTaskResults(cls, taskTemplates: List[EPMTaskTemplate]) -> GetTaskResultsResponse:
        """
        Return the task results of a given task template
        """
        return cls.execute_soa_method(
            method_name='getTaskResults',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='Workflow',
            params={'taskTemplates': taskTemplates},
            response_cls=GetTaskResultsResponse,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def setRichContent(cls, inputs: List[SetContentInput]) -> ServiceData:
        """
        This SOA operation sets the body text property of a FullText object to the supplied html or pain text content.
        The SOA operation is called when a user modifies requirements in the active workspace client. It accepts
        SpecElement Revision or FullText objects to process. All modified objects and exceptions are added to the
        returned service data.
        """
        return cls.execute_soa_method(
            method_name='setRichContent',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData]) -> ExportToApplicationResponse:
        """
        exportToApplication
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse,
        )


class LOVService(TcService):

    @classmethod
    def validateLOVValueSelections(cls, lovInput: LOVInput, propName: str, uidOfSelectedRows: List[str]) -> ValidateLOVValueSelectionsResponse:
        """
        This operation can be invoked after selecting a value from the LOV.  Use this operation to do additional
        validation to be done on server such as validating Range value, getting the dependent properties values in case
        of interdependent LOV (resetting the dependendent property values), Coordinated LOVs ( populating dependent
        property values )
        """
        return cls.execute_soa_method(
            method_name='validateLOVValueSelections',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='LOV',
            params={'lovInput': lovInput, 'propName': propName, 'uidOfSelectedRows': uidOfSelectedRows},
            response_cls=ValidateLOVValueSelectionsResponse,
        )

    @classmethod
    def getInitialLOVValues(cls, initialData: InitialLovData) -> LOVSearchResults:
        """
        This operation is invoked to query the data for a property having an LOV attachment. The results returned from
        the server also take into consideration any filter string that is in the input.  This operation returns both
        LOV meta data as necessary for the client to render the LOV and partial LOV values list as specified.
        
        The operation will return the results in the LOVSearchResults data structure. Maximum number of results to be
        returned are specified in the InitialLOVData data structure. If there are more results, the moreValuesExist
        flag in the LOVSearchResults data structure will be true. If the flag is true, more values can be retrieved
        with a call to the getNextLOVValues operation.
        """
        return cls.execute_soa_method(
            method_name='getInitialLOVValues',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='LOV',
            params={'initialData': initialData},
            response_cls=LOVSearchResults,
        )

    @classmethod
    def getNextLOVValues(cls, lovData: LOVData) -> LOVSearchResults:
        """
        This operation is invoked after a call to getInitialLOVValues if the moreValuesExist flag is true in the
        LOVSearchResults output returned from a call to the getInitialLOVValues operation. The operation will retrieve
        the next set of LOV values
        """
        return cls.execute_soa_method(
            method_name='getNextLOVValues',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='LOV',
            params={'lovData': lovData},
            response_cls=LOVSearchResults,
        )


class FinderService(TcService):

    @classmethod
    def findObjectsByClassAndAttributes2(cls, input: FindObjectsInput2) -> FindObjectsResponse2:
        """
        Returns a list of objects for a specific class type with certain attributes and values.
        """
        return cls.execute_soa_method(
            method_name='findObjectsByClassAndAttributes2',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='Finder',
            params={'input': input},
            response_cls=FindObjectsResponse2,
        )

    @classmethod
    def findUsersTasks(cls) -> FindUsersTasksResponse:
        """
        Finds all the tasks the user has.
        """
        return cls.execute_soa_method(
            method_name='findUsersTasks',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='Finder',
            params={},
            response_cls=FindUsersTasksResponse,
        )


class OrganizationManagementService(TcService):

    @classmethod
    def getGroupMembership(cls, groupMembershipInput: GroupMembershipInput) -> GroupMembershipResponse:
        """
        This operation returns a list of business objects containing group membership information.
        """
        return cls.execute_soa_method(
            method_name='getGroupMembership',
            library='Internal-AWS2',
            service_date='2012_10',
            service_name='OrganizationManagement',
            params={'groupMembershipInput': groupMembershipInput},
            response_cls=GroupMembershipResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindStudiesResponse(TcBaseObj):
    """
    The response data from the findStudies service operation.
    
    :var results: The found Mfg0BvrStudy objects for each inputNodes Mfg0BvrProcess or Mfg0BvrOperation
    :var serviceData: Service Data
    """
    results: List[FindStudiesResults] = ()
    serviceData: ServiceData = None


@dataclass
class FindStudiesResults(TcBaseObj):
    """
    A list of Mfg0BvrStudy objects
    
    :var listOfStudies: A list of Mfg0BvrStudy objects
    """
    listOfStudies: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.Vendormanagement._2012_09.VendorManagement import VendorRoleData


class VendorManagementService(TcService):

    @classmethod
    def addRemoveVendorRoles(cls, vendorRoles: List[VendorRoleData], vendorRevRef: BusinessObject) -> ServiceData:
        """
        This operation adds or removes set of roles to/from a VendorRevision. Typically a role can be assigned to a
        VendorRevision by adding a role form with different role specific information. OOTB VendorRevision can be
        assigned with 3 roles namely Distributor, Supplier and Manufacturer. All these role forms contain information
        about the Vendor based on the type of the role. Similarly a role can be revoked from a VendorRevision. When a
        Vendor is revised, the new revision inherits all the assigned roles to the previous VendorRevision.
        """
        return cls.execute_soa_method(
            method_name='addRemoveVendorRoles',
            library='Vendormanagement',
            service_date='2012_09',
            service_name='VendorManagement',
            params={'vendorRoles': vendorRoles, 'vendorRevRef': vendorRevRef},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.ImportExport._2011_06.FileImportExport import ExportToApplicationInputData2, MarkupReqResponse, ImportFromApplicationInputData2, ExportToApplicationResponse1, GetExportTemplateResponse1, MarkupReqInput, GetTemplateInput, MarkupReqUpdateResponse, MarkupReqUpdateInput, ImportFromApplicationResponse1
from typing import List
from tcsoa.utils import TcService


class FileImportExportService(TcService):

    @classmethod
    def importFromApplication(cls, inputs: List[ImportFromApplicationInputData2]) -> ImportFromApplicationResponse1:
        """
        This operation is used for importing the contents of the given MSWord  document (.docx) to create objects of
        type SpecElement.  Based on the application format, this operation can also be used for importing SpecTemplate,
        ObjectTemplate and ExcelTemplate in database. 
        If the application format is "MSWordXML" then the operation parses the given MS Word document to creates a
        structure of type SpecElement.
         If the application format is "MSWordSpecTemplate" or "MSWordObjTemplate" or "MSExcelTemplate" then the chosen
        template can be imported into Teamcenter. 
        If the application format is "MSWordXMLLive" then a "Live" document can be imported to Teamcenter.  To use this
        application format, a "Live" document should be generated from Teamcenter.
        If the application format is "MSWordSetContent" then it can set the rich text of the SpecElement  by applying
        the templates at the server.
        If the application format is "MSWordXMLExisting" then the given MSWord document is imported to Teamcenter to
        create a Specification within an existing chosen Specification.
        If the application format is "MSWordImportUsingKW" then the given MSWord document is imported to Teamcenter if
        user chooses keywords supplied by user. The keywords are parsed at the server to create the appropriate
        SpecElement type when the keyword is found.
        If the application format is "StructureOnly" then the given MSWord Live document is imported to Teamcenter
        without its  contents.(only object_name property value)
        If the application format is "StructureWithContents" then the given MSWord Live document is imported to
        Teamcenter along with its rich text contents.
        If the application format is "MSWordXWithFulltext" then this operation can be used to create new FullText with
        provided rich text in given MSWord document and this FullText needs to be attached to selected ItemRevision.
        Currently this gets called from Rich client in case of editing rich text for only Systems Engineering revision
        objects,if there is no FullText dataset associated.to selected ItemRevision.
        If the application format is "MSWordXWithFulltext"then this operation can be used to create FullText object
        with input rich text in given document. The created FullText object will be attached to selected object
        revision. Currently this format is called from Rich client in case of editing rich text for only Systems
        Engineering revision objects and if there is no FullText dataset associated.to selected revision.
        When Objects objects of type Requirement/Paragraph are created at the server after importing the document. 
        When this operation is called from Teamcenter rich client, a structure is created and is opened in the
        RequirementsManagement application.  The structure and indentation of the child Requirement is driven by the
        outline levels in the document.  Each Paragraph that is formatted in an outline level style produces a separate
        Requirement. This file may be located in a local file system folder or a network folder.  Import of document to
        Teamcenter supports Word documents in MSOffice Open XML format (.docx format) only. Import of templates to
        Teamcenter supports Word documents in MSOffice Open XML format. (.docx and .docm format)
        
        Supported application formats for this operation
        
        - StructureOnly    This format is used for importing MSWord document to Teamcenter without its  contents.(only
        object_name property value).
        - StructureWithContents This format is used for importing WorkspaceObject to MSWord Live and so that user can
        modify its contents and save them back to Teamcenter.
        - MSWordImportUsingKW    This format is used for importing a MSWord document to Teamcenter using keywords
        supplied by user.
        - MSWordImportExistingUsingKW This format is used for importing a MSWord document to create a Specification in
        Teamcenter into an existing Specification when keywords are supplied by user.
        - MSWordXWithFulltext    This format is used for importing a MSWord document when user wanted to create new
        FullText with provided rich text in given document and this FullText needs to be attached to selected
        ItemRevision. Currently this gets called from Rich client in case of editing rich text for only ItemRevision
        objects, if there is no FullText dataset associated to selected ItemRevision.
        - MSWordXMLPreview This format is used for importing a MSWord document to create a Specification from a
        document previewed by Office client.
        
        
        
        Use cases:
        User can create an MSWord document and import it using this operation to create a structure. Each paragraph in
        Word document represents a Requirement/Paragraph in the structure. User can import templates (SpecTemplate,
        ObjectTemplate and ExcelTemplate) to Teamcenter. User can import a "Live" document generated in Teamcenter and
        set the properties of Workspace objects. User can set the rich text of the SpecElement using the embedded
        MSWord from RAC. User can prevent overwrite of objects which are edited by others.
        """
        return cls.execute_soa_method(
            method_name='importFromApplication',
            library='ImportExport',
            service_date='2011_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse1,
        )

    @classmethod
    def updateReqMarkup(cls, inputs: List[MarkupReqUpdateInput]) -> MarkupReqUpdateResponse:
        """
        This operation creates new markups and updates existing markups in rich text content of SpecElement objects
        that are exported to MSWord using markup option. 
        This operation will take information about markups in the form of mrk file. User needs to have Office Client
        installed to create and update the markups in Word document.
        
        Use cases:
        User can install Office Client and export SpecElement to MSWord using the markup option. Office Client provides
        a mechainsm to create or update markups in a Word document.
        """
        return cls.execute_soa_method(
            method_name='updateReqMarkup',
            library='ImportExport',
            service_date='2011_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=MarkupReqUpdateResponse,
        )

    @classmethod
    def createReqMarkup(cls, inputs: List[MarkupReqInput]) -> MarkupReqResponse:
        """
        This operation will take info for markup datasets to be created for the requirement objects and create a markup
        dataset for each markup data requirement object
        """
        return cls.execute_soa_method(
            method_name='createReqMarkup',
            library='ImportExport',
            service_date='2011_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=MarkupReqResponse,
        )

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData2]) -> ExportToApplicationResponse1:
        """
        This operation is used for exporting Teamcenter objects (WorkspaceObject) to applications like MSWord and
        MSExcel. Teamcenter business objects should already be created so that the objects can be exported to Word and
        Excel using the service operation.
        An MSExcel (.xlsm) or a MSWord (.docx) file or a comma separated file (.csv) file is generated at the server as
        a result of this operation.  The generated file will contain Teamcenter objects and their properties. Depending
        upon the application format that is being passed as input parameter, the generated file can be opened in MSWord
        or MSExcel.  If the application format is "'CSV'" then a comma separated file is generated at the server.  If
        the application format is "'MSWordXML'" then the generated document is a Word document.  It traverses the
        structure for the given SpecElement deep and exports all its children to MSWord document. If the application
        format is "'MSExcel'" then the generated sheet is a static Excel spreadsheet.  If the application format is
        "'MSExcelLive'" then the generated sheet is a Live Excel spreadsheet. "Live" sheet means that modifications can
        be done to the data will reflect to Teamcenter.  If the application format is "'MSWordXMLLive'" then the
        generated document is a Live Word document.  This mode provides the capability of Live editing the SpecElement
        in the word document.  A "Live" Word document means that the any modifications done in the document like
        changing the rich text or editing of properties will be saved to Teamcenter on click of "Save"button in MSWord.
        If the application format is "'MSExcelReimport'" then the generated sheet can be reimported back to Teamcenter.
        If the application format is "'MSExcelLiveBulkMode'" then the sheet is generated in "Bulk Live" mode. This mode
        enables the user to make all the changes to the properties of objectsfrom Excel and then commit the changes to
        Teamcenter on click of "Save to Teamcenter" button in Excel.  If the application format is "'StructureOnly'"
        then the structure can be exported to Word without the contents having only "object_name" property in the
        exported document.  If the application format is "'StructureWithContents'" then the structure is exported to
        MSWord along with the contents (rich text) of each element in the structure.  This mode provides the ability of
        live editing and structure editing of SpecElements in the MSWord document. User can edit the contents or can
        make structural changes to the exported document.  If the application format is "'MSWordXMLLiveMarkupFN'" then
        Markups will be exported to MSWord using FindNo as sorting key.  If the application format is "'MSWordXMLFN'"
        then a static structure can be exported to MSWord using FindNo as sorting key.  If the application format is
        "'MSWordXMLLiveFN'" then a "Live" structure can be exported to MSWord using FindNo as sorting key.  If the
        export Option is "CheckOutObjects" then the objects can be checked out before exporting to MSWord/Excel.
        
        Use cases:
        User can create Teamcenter objects in RAC and then export those objects and their properties to Word or Excel.
        If "Live" option is selected then user can create "Live" documents after export to Word/Excel.  During the
        export to Word/Excel, Teamcenter objects are exported to Word/Excel by applying the appropriate templates. User
        can export Teamcenter objects to Excel which can be imported back to Teamcenter. User can create excel sheet in
        a "Bulk Live" mode so that bulk changes can be committed to Teamcenter.  User can export the objects and
        properties to a comma separated file. User can create SpecTemplate, ObjectTemplate and ExcelTemplate in
        Teamcenter and customize these templates as per User need such as adding more properties or adding formatting
        information to the templates.  User can associate different objectTemplate for every Business object type at
        runtime and specify this configuration when using this operation. User can export a structure to MSWord without
        exporting the rich text of each SpecElement. User can export a structure for "Live" edit and structure edit to
        MSWord.  User can export a static or a "Live" structure along with the Markups. Users can checkout objects
        during Export to MSWord/MSExcel.
        
        Exceptions:
        >Service Exception If there is any error during generating transient file read ticket due to a configuration
        issue at the server, then the operation throws a service exception. Example- If the transient volume directory
        is not configured at the server then the FMS fails to generate a file at the server and subsequent file
        download operation fails. In such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='ImportExport',
            service_date='2011_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse1,
        )

    @classmethod
    def getExportTemplates(cls, filter: List[GetTemplateInput]) -> GetExportTemplateResponse1:
        """
        'getExportTemplates' is responsible for getting various export templates from database based on input filter.
        Export templates can be of type SpecTemplate, ObjectTemplate or ExcelTemplate. Depending upon the input filter,
        the templates of type SpecTemplate or ObjectTemplate or ExcelTemplate are retrieved from the database. These
        templates can be used by the application for export purpose.
        
        Use cases:
        User can create Items of type SpecTemplate or ObjectTemplate or ExcelTemplate in Teamcenter and use this
        operation to get the desired template types.
        """
        return cls.execute_soa_method(
            method_name='getExportTemplates',
            library='ImportExport',
            service_date='2011_06',
            service_name='FileImportExport',
            params={'filter': filter},
            response_cls=GetExportTemplateResponse1,
        )



from tcsoa.gen.Internal_Administration._2013_05.services import UserManagementService as imp0
from tcsoa.gen.Internal_Administration._2014_10.services import UserManagementService as imp1
from tcsoa.gen.Internal_Administration._2018_11.services import SiteManagementService as imp2
from tcsoa.gen.Internal_Administration._2007_06.services import PreferenceManagementService as imp3
from tcsoa.gen.Internal_Administration._2018_06.services import PreferenceManagementService as imp4
from tcsoa.gen.Internal_Administration._2008_06.services import IRMService as imp5
from tcsoa.gen.Internal_Administration._2016_09.services import VolumeManagementService as imp6
from tcsoa.gen.Internal_Administration._2017_11.services import IRMService as imp7
from tcsoa.gen.Internal_Administration._2007_06.services import AuthorizationService as imp8
from tcsoa.gen.Internal_Administration._2019_06.services import UserManagementService as imp9
from tcsoa.gen.Internal_Administration._2012_10.services import OrganizationManagementService as imp10
from tcsoa.gen.Internal_Administration._2011_06.services import OrganizationManagementService as imp11
from tcsoa.gen.Internal_Administration._2009_10.services import PersonManagementService as imp12
from tcsoa.gen.Internal_Administration._2015_10.services import UserManagementService as imp13
from tcsoa.utils import TcService


class UserManagementService(TcService):
    activateUsers2 = imp0.activateUsers2
    activateUsers3 = imp1.activateUsers2
    getGroupRoleViewModelRows = imp9.getGroupRoleViewModelRows
    resetUserPassword = imp13.resetUserPassword


class SiteManagementService(TcService):
    createOrUpdateSites = imp2.createOrUpdateSites


class PreferenceManagementService(TcService):
    createPreferenceCategories = imp3.createPreferenceCategories
    deletePreferenceCategories = imp4.deletePreferenceCategories
    deletePreferences = imp3.deletePreferences
    exportPreferences = imp3.exportPreferences
    getModifiedSitePreferences = imp3.getModifiedSitePreferences
    getNonSessionPreferences = imp3.getNonSessionPreferences
    importPreferences = imp3.importPreferences


class IRMService(TcService):
    getACLsByType = imp5.getACLsByType
    getAccessorsInfo = imp7.getAccessorsInfo


class VolumeManagementService(TcService):
    getAccessibleVolumes = imp6.getAccessibleVolumes


class AuthorizationService(TcService):
    getAuthorization = imp8.getAuthorization
    setAuthorization = imp8.setAuthorization


class OrganizationManagementService(TcService):
    getOrganizationGroupMembers = imp10.getOrganizationGroupMembers
    getOrganizationGroups = imp11.getOrganizationGroups


class PersonManagementService(TcService):
    getStringProperties = imp12.getStringProperties
    setStringProperties = imp12.setStringProperties



from __future__ import annotations

from tcsoa.gen.Core._2019_06.DataManagement import DeleteIn
from tcsoa.gen.Core._2019_06.Session import LicAdminInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SessionService(TcService):

    @classmethod
    def licenseAdmin(cls, licAdminInput: List[LicAdminInput]) -> ServiceData:
        """
        This operation provides licensing related operations such as check-out and check-in of a license feature key.
        """
        return cls.execute_soa_method(
            method_name='licenseAdmin',
            library='Core',
            service_date='2019_06',
            service_name='Session',
            params={'licAdminInput': licAdminInput},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def unlinkAndDeleteObjects(cls, deleteInput: List[DeleteIn]) -> ServiceData:
        """
        This operation unlinks the input objects from their corresponding container and then attempts to deletes them.
        The input objects are related to the container as the reference or relation property supplied as part of the
        input. The operation also takes a flag whether to unlink the objects from the container in case the deletion
        fails.
        
        After unlinking the objects from the input container, if the objects being deleted are still referenced by
        other objects then error is returned to the caller. Any other error in deletion of the objects are also
        returned to the caller.
        
        In case the input argument objectsToDelete contains objects of type Item, then the operation also deletes all
        ItemRevision objects, associated ItemMaster, ItemRevisionMaster form objects.
        
        If the input argument objectsToDelete are of type ItemRevision and if is the last revision of the Item then the
        operation deletes the Item, associated ItemMaster, ItemRevisionMaster form objects.
        
        The input argument objectsToDelete can be an ImanRelation.
        """
        return cls.execute_soa_method(
            method_name='unlinkAndDeleteObjects',
            library='Core',
            service_date='2019_06',
            service_name='DataManagement',
            params={'deleteInput': deleteInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Manufacturing._2014_12.StructureSearch import AdditionalInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindCPCResponse(TcBaseObj):
    """
    List of CPC objects found.
    
    :var foundCPCs: The list of MECollaborationContext objects.
    :var serviceData: The service data containing partial errors if any.
    """
    foundCPCs: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class FindEquivalentLinesIn(TcBaseObj):
    """
    Contains the details of an equivalence search input.
    
    :var sourceLines: The source BOMLine objects to start the equivalent search from.
    :var targetScopes: The BOMLine objects representing target scopes to search equivalents under.
    :var isSourceAScope: If true, sourceLines represent a list of scopes under which traversal will be performed.
    Otherwise, sourceLines are those lines that themselves will be searched for.
    :var sourceClosureRule: If isSourceAScope is true, this field is the closure rule name to traverse the source
    scopes.
    :var searchCriteria: The search criteria. Currently supported values: 1 - ID in Context (Top Level) only, 2 - ID in
    Context (Top Level) or Item ID and Absolute Transformation Matrix.
    :var additionalInfo: Additional input information; currently not used.
    """
    sourceLines: List[BusinessObject] = ()
    targetScopes: List[BusinessObject] = ()
    isSourceAScope: bool = False
    sourceClosureRule: str = ''
    searchCriteria: int = 0
    additionalInfo: AdditionalInfo = None


@dataclass
class FindEquivalentLinesResp(TcBaseObj):
    """
    Contains the details of the equivalence search results.
    
    :var equivalenceResults: A list of EquivalenceResult objects containing the equivalence relation between source
    BOMLine objects and their equivalent target BOMLine objects. The size of equivalenceResults list in this structure
    matches that of the searchInputs list.
    :var serviceData: The ServiceData containing partial errors.
    :var additionalInfo: Additional input information; currently not used.
    """
    equivalenceResults: List[EquivalenceResult] = ()
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class EquivalenceResult(TcBaseObj):
    """
    Contains the details of equivalence search result for a single FindEquivalentLinesIn object.
    
    :var equivalenceResults: A map from source BOMLine objects to their equivalent target BOMLine objects. The size of
    the map matches that of the sourceLines list in the FindEquivalentLinesIn structure.
    """
    equivalenceResults: ObjToObjVectorMap = None


"""
A map of BusinessObject to vector of BusinessObject instances.
"""
ObjToObjVectorMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ProjectManagement._2008_06.ScheduleManagement import ScheduleModifyResponses, ScheduleModifyContainer, MultipleScheduleLoadResponses
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def getSchedulesToinsert(cls, masterSchedule: Schedule) -> ServiceData:
        """
        The operation is used to query for all schedules where the user is a ScheduleMember  object with schedule
        manager coordinator privilege. After querying for the schedules, the business logic filters the list to ensure
        that schedules in the list that is returned do not cause circular dependencies when used in a master schedule.
        The operation returns the list of schedules in the 'ServiceData'  as 'm_plainObjs'. By design the list is meant
        to be inserted into a designated master schedule. 
        """
        return cls.execute_soa_method(
            method_name='getSchedulesToinsert',
            library='Internal-ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'masterSchedule': masterSchedule},
            response_cls=ServiceData,
        )

    @classmethod
    def loadSchedules(cls, schedule: List[str]) -> MultipleScheduleLoadResponses:
        """
        Loads all the objects in a schedule needed for the scheduling logic.
        """
        return cls.execute_soa_method(
            method_name='loadSchedules',
            library='Internal-ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'schedule': schedule},
            response_cls=MultipleScheduleLoadResponses,
        )

    @classmethod
    def modifySchedule(cls, scheduleModifyContainers: List[ScheduleModifyContainer]) -> ScheduleModifyResponses:
        """
        Updates all the affected scheduling objects based on the initial users request to the application interface. 
        Schedule properties can be updated. New schedule tasks, dependencies and resource assignments can be created
        through this operation. Existing schedule tasks, dependencies and resource assignments can be updated. Existing
        schedule tasks, dependencies and resource assignments can be deleted.
        
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='modifySchedule',
            library='Internal-ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'scheduleModifyContainers': scheduleModifyContainers},
            response_cls=ScheduleModifyResponses,
        )



from tcsoa.gen.Internal_Translation._2007_06.services import TranslationManagementService as imp0
from tcsoa.utils import TcService


class TranslationManagementService(TcService):
    createDatasetOfVersion = imp0.createDatasetOfVersion
    insertDatasetVersion = imp0.insertDatasetVersion
    queryTranslationRequests = imp0.queryTranslationRequests
    updateTranslationRequest = imp0.updateTranslationRequest



from __future__ import annotations

from typing import List
from tcsoa.gen.Visualization._2016_03.DataManagement import MetaDataStampInputInfo, MetaDataStampOutputResponse
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getMetaDataStampWithContext(cls, metaDataStampInfos: List[MetaDataStampInputInfo]) -> MetaDataStampOutputResponse:
        """
        This operation retrieves metadata stamp files for Teamcenter objects based on the context within which these
        objects are being displayed.  For example, when a user prints a model view, the stamp will vary based on the
        context within which this model view is printed.  Printing the model view in owning model context provides one
        stamp, while printing the same view from within a disclosure context produces another stamp.
        
        The metadata stamp is generated based on the list of objects and the context of those objects using the
        metaDataStampInfos list. The transient ticket for the generated *.mds file is returned. The ticket can then be
        used to retrieve the file using a FMS method like FccProxy::downloadFiles. 
        
        The Teamcenter default implementation uses a Metadatastamp template containing an MDS file with Teamcenter
        property names and default values. The Teamcenter site preference MetaDataStamp_template is used to find the
        item where the template files is stored. This file is processed and each Attribute specified will be replaced
        with the matching object property values if found. The output mds file will be written to the transient volume
        and the transient ticket of the file is sent to client in the response data. The customization hook provided
        with this interface may wish to use the same template mds file as a starting point.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        Use cases:
        This method is called by visualization when integrating with Teamcenter for printing objects like
        Fnd0ModelViewProxy and SnaphotViewData datasets that are loaded into some higher level product context. The
        user loads a configured product structure, or some high level object that sets context in Teamcenter. Within
        this structure or context, the user finds objects they wish to print, and invokes a print action, The stamping
        context is the higher level context within which the stamping metadata is retrieved from (e.g. the root node of
        the product structure), and the stamped objects are the individual objects that are to be printed (e.g. model
        views or product Views). Stamp files (tickets) are returned to the caller, and these files are used by
        visualization to place metadata markings on views printed. The stamp file represents a metadata stamp overlay
        on top of the printed image and often contains control information for printed detailed design data.
        
        The MDS file created by this service can also be used by the VisView Convert and Print utilities.
        
        This service may also be useful for customizations that leverage Siemens Embedded visualization toolkits, and
        thus it is a public service.
        """
        return cls.execute_soa_method(
            method_name='getMetaDataStampWithContext',
            library='Visualization',
            service_date='2016_03',
            service_name='DataManagement',
            params={'metaDataStampInfos': metaDataStampInfos},
            response_cls=MetaDataStampOutputResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMWindow
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ResourceManagementService(TcService):

    @classmethod
    def showCoordinateSystems(cls, bomWindow: BOMWindow, showCoordinateSystems: bool) -> ServiceData:
        """
        In the Resource Manager application, coordinate systems are used for automatic positioning of components during
        the assembly process. Those coordinate systems are displayed in the BOM tree table in the Resource view in the
        Teamcenter rich client. In some cases the user wants to focus on the main components and hide the coordinate
        systems. This operation changes the configuration to show or hide the coordinate system of BOMLine objects in a
        specific BOMWindow.
        """
        return cls.execute_soa_method(
            method_name='showCoordinateSystems',
            library='Internal-Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'bomWindow': bomWindow, 'showCoordinateSystems': showCoordinateSystems},
            response_cls=ServiceData,
        )

    @classmethod
    def showGCSConnectionPoints(cls, bomWindow: BOMWindow, showGCSConnectionPointsOption: int) -> ServiceData:
        """
        In the Resource Manager application, connection points are used for the guided component search (GCS) of
        components during the assembly process. Those connection points are displayed in the BOM tree table in the
        Resource view in the Teamcenter rich client. In some cases, the user wants to focus on the main components and
        hide the connection points. This operation can be used to show or hide the connection point BOMLine objects in
        a specific BOMWindow.
        In addition to hide (0) and show (1), a third option "only unconnected" (2) can be used. If this option is used
        only those connection points are displayed that can be used for assembling and that are not yet connected to
        another component.
        """
        return cls.execute_soa_method(
            method_name='showGCSConnectionPoints',
            library='Internal-Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'bomWindow': bomWindow, 'showGCSConnectionPointsOption': showGCSConnectionPointsOption},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import WorkspaceObject, TC_Project
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class Filter(TcBaseObj):
    """
    This is a list of strings to support both single valued and multi valued properties of any type. The calling client
    is responsible for converting the different property types (like integer, double, date etc...) to a string using
    the appropriate toXXXString functions in the Services client framework's Property class.
    
    :var sourceTypeName: The name of the WorkspaceObject and or any of its subtype  on which filter is to be performed.
    :var name: The name of the attribute on a given type.
    :var value: The value of the attribute.
    """
    sourceTypeName: str = ''
    name: str = ''
    value: str = ''


@dataclass
class GetFilteredProjectDataInputData(TcBaseObj):
    """
    Object that holds project id and a list  of filters.
    
    :var projectID: The project id of the  TC_Project object.
    :var filters: A list of Filter critera objects.
    """
    projectID: str = ''
    filters: List[Filter] = ()


@dataclass
class GetFilteredProjectDataOutput(TcBaseObj):
    """
    Object that holds project ID and filtered data.
    
    :var projectID: The project id of  the TC_Project object.
    :var filteredData: A list of filtered project  data.
    """
    projectID: str = ''
    filteredData: List[WorkspaceObject] = ()


@dataclass
class GetFilteredProjectDataResponse(TcBaseObj):
    """
    Object that holds vector of GetFilteredProjectDataOutput and standard ServiceData.
    
    :var output: A list of  GeFilteredProjectDataOutput objects one for each project.
    :var serviceData: A standard ServiceData.
    """
    output: List[GetFilteredProjectDataOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ProjectSmartFolderHierarchyOutput(TcBaseObj):
    """
    This structure holds the project ID of the project and a list of SmartFolderOutputInfo structures.
    
    :var projectID: The project id of TC_Project object.
    :var projectSmartFolderHierarchy: A list containing information about smart folder hierarchies associated with the
    project ids.
    """
    projectID: str = ''
    projectSmartFolderHierarchy: List[SmartFolderOutputInfo] = ()


@dataclass
class ProjectSmartFolderHierarchyOutputResponse(TcBaseObj):
    """
    This structure contains the list of ProjectSmartFolderHierarchyOutput objects and standard ServiceData.
    
    :var output: A list of ProjectSmartFolderHierarchyOutput objects which contains  project IDs and Smart folder
    hierarchy information.
    :var serviceData: Holds the error list corresponding to the project ids for which smart folder hierarchy retrieval
    failed.
    """
    output: List[ProjectSmartFolderHierarchyOutput] = ()
    serviceData: ServiceData = None


@dataclass
class SmartFolderOutputInfo(TcBaseObj):
    """
    This structure holds the information retrieved corresponding to one top level smart folder.
    
    :var parentName: The name of the parent node of the current smart folder hierarchy .
    :var name: The name of the current node.
    :var filters: A list of filters for this hierarchy.
    :var isLeaf: The flag indicating if this is the last child of the hierarchy.
    """
    parentName: str = ''
    name: str = ''
    filters: List[Filter] = ()
    isLeaf: bool = False


@dataclass
class TopLevelHierarchyOutputResponse(TcBaseObj):
    """
    This structure contains the top-level hierarchy as a flat list in vector format but in a depth first manner. Each
    SmartFolderOutputInfo in the vector would correspond to a node in the hierarchy and would have a parent smart
    folder name under which it belongs.
    
    :var topHierarchy: List of SmartFolderOutputInfo structures.
    :var userProjects: List of UserProjectsOutput structures.
    :var serviceData: A  standard ServiceData which contains unknown errors.
    """
    topHierarchy: List[SmartFolderOutputInfo] = ()
    userProjects: List[UserProjectsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class UserProjectsOutput(TcBaseObj):
    """
    This structure would contain the current logged in users project along with the filters defined on those projects.
    
    :var project: The TC_Project for the current user.
    :var filters: A list of filters for the project.
    """
    project: TC_Project = None
    filters: List[Filter] = ()



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ObjectsForOwnershipTransferResponse(TcBaseObj):
    """
    It holds FMS ticket of report file which lists candidates for ownership transfer and errors if any.
    
    :var fileFmsTickets: Output file (report) ticket
    :var serviceData: The service data
    """
    fileFmsTickets: str = ''
    serviceData: ServiceData = None


@dataclass
class TransferOwnershipResponse(TcBaseObj):
    """
    It holds the FMS ticket of report file which has information about the status of transfer ownership operation.
    
    :var fileFmsTickets: Output file (report) ticket
    :var serviceData: The service data
    """
    fileFmsTickets: str = ''
    serviceData: ServiceData = None


@dataclass
class UpdateOwnershipTransferResponse(TcBaseObj):
    """
    It holds the FMS ticket of report which has information about the status of update ownership operation and errors
    if any.
    
    :var fileFmsTickets: Output file (report) ticket
    :var serviceData: The service data
    """
    fileFmsTickets: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0AbsFamilyGroup, Cfg0AbsOptionValue, Cfg0AbsFamily, POM_object, Cfg0AbsAllocation, Cfg0AbsAssociation, Cfg0SearchCursor, BusinessObject, Cfg0AbsOptionFamily, Cfg0AbsValue, Cfg0ConfiguratorPerspective, TransferOptionSet, ImanQuery, WorkspaceObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ValidateProductConfigOutput(TcBaseObj):
    """
    The output structure with the given input 'BusinessObjectConfigExpression' to 'validateProductConfiguration'
    operation and the list of violations for the given BusinessObjectConfigExpression. Each 'Volation' structure
    provides information about the rule which is violate with its severity and the error message.
    
    :var updatedExpression: Updated 'BusinessObjectConfigExpression' with values applied from inclusion rules of the
    given product item.
    :var violations: List of 'Violation' structures with rules and error messages.
    """
    updatedExpression: BusinessObjectConfigExpression = None
    violations: List[Violation] = ()


@dataclass
class ValidateProductConfigurationResponse(TcBaseObj):
    """
    The response structure for the operation 'ValidateProductConfiguration'
    
    :var serviceData: The service data for errors and returned objects.
    :var outputs: The list of expressions that was validated and updated with selections due to application of
    constraints.
    """
    serviceData: ServiceData = None
    outputs: List[ValidateProductConfigOutput] = ()


@dataclass
class ValueFilters(TcBaseObj):
    """
    A filter filtering rules for all features specified in this structure.
    
    :var valueIDs: List of value IDs (' cfg0object_id' ). If specified all values with matching criteria will be
    searched and will be used to filter rules. This parameter is mutually exclusive of 'values'.
    :var values: List of values. If specified values will be used to filter rules. This parameter is mutually exclusive
    of 'valueIds'.
    """
    valueIDs: List[str] = ()
    values: List[Cfg0AbsOptionValue] = ()


@dataclass
class VariantInfo(TcBaseObj):
    """
    A SOA structure representation of 'Cfg0BaseConfigurator'::'VaraintInfo'. This structure can be viewed as the
    serialization of the cache of variant data stored as a part of 'Cfg0BaseConfigurator'::'VaraintInfo'.
    
    :var lastUpdate: Last updated time of cache. Use this value to decide whether cache needs to be refreshed.
    :var variantOption: A collection of the option families and thier values associated with the variant cache of the
    item revision.
    :var validValueRanges: Collection of free from familiy value ranges associated with the item revision.
    :var defaultConditions: A collection of Expression (wrapper over Math::BooleanExpression) objects associated with
    the variant data. Default conditions are the combined expression of modelCondition, if any, and
    applicabilityCondition, if any, for each default rule, which need to be satisfied before the default value is
    applied to the expression.
    :var defaultExpressions: A collection of Expression (wrapper over Math::BooleanExpression) objects. Default
    expressions are the subject part of a default rules that would be applied on the input expression if default
    conditions are satisfied (earlier called derived defaults) or even without them (earlier called fixed defaults).
    :var ruleChecks: A collection of ConfiguratorConstraint objects that contain inclusion and exclusion rules defined
    on the variant data for this item revision.
    :var familyHash: The Map of pairs ( Cfg0AbsFamily, 'VariantOption' ) representing the family instance and its
    structure form.
    :var familyNameHash: A data structure comprising of map between the family name (as a key) and its family
    information (representation Math::VariantOption) as the value. Family name is used as the key in this map as
    against the fully qualified name used in familyHash.
    :var ruleCheckExpressions: Rule check expression is the combined boolean expression of all the inclusion and
    exclusion rules defined on the variant data associated with this item revision.
    """
    lastUpdate: datetime = None
    variantOption: List[VariantOption] = ()
    validValueRanges: List[Expression] = ()
    defaultConditions: List[Expression] = ()
    defaultExpressions: List[Expression] = ()
    ruleChecks: List[ConfiguratorConstraint] = ()
    familyHash: FamilyHash = None
    familyNameHash: FamilyNameHash = None
    ruleCheckExpressions: Expression = None


@dataclass
class VariantOption(TcBaseObj):
    """
    Varaint option is a wrapper over Math::VariantOption, an internal representation that holds the variant data
    associated with this item revision. Data from 'Math'::'VariantOption' would be transfered into the 'VariantOption'
    SOA structure during serialization and converted back into 'Math'::'VaraintOption' during de-serialization.
    
    :var optionNamespace: Option namespace
    :var name: Name of the option
    :var description: User entered description value of this option.
    :var tag: Concatenated value of the format dict.familyname{operator}value
    :var valueDataType: Data type of this family. Valid values for this parameter are defined by the ListOfValues
    "Cfg0FamilyValueDataType".
    :var uom: Unit of measure associated with this option.
    """
    optionNamespace: str = ''
    name: str = ''
    description: str = ''
    tag: str = ''
    valueDataType: str = ''
    uom: str = ''


@dataclass
class VariantRuleInput(TcBaseObj):
    """
    A structure that provides information required to create or update a variant rules.
    
    :var expressions: The list of variant expressions to set on the variant rule.
    :var saveRule: Indicates whether the rule should be saved before returning. If the value is FALSE a transient
    variant rule will be returned. Transient variant rules and transient variant rule changes are discarded at the end
    of the Teamcenter session unless they are explicitly saved in a separate operation.
    :var ruleToUpdate: If specified the expression on this rule is updated with the input expression; to create a new
    rule this param is set to null.
    :var creInputs: A map that ties properties to their values for the new rule.
    :var relationName: The relationship name to use when associating the variant rule to the object specified in
    parameter "'referenceObject'". This is an optional parameter. If specified a relationship of this type is used to
    associate the variant rule to the reference object. A NULL value causes the default relationship name to be used as
    specified in preference "TC_Default_SVR_Relationship".
    :var referenceObject: When specified the newly created rule is related to the given reference object.
    """
    expressions: List[ConfigExpression] = ()
    saveRule: bool = False
    ruleToUpdate: POM_object = None
    creInputs: CreateInput = None
    relationName: str = ''
    referenceObject: WorkspaceObject = None


@dataclass
class VariantRuleOutput(TcBaseObj):
    """
    Output structure for each 'VariantRuleInput' in 'createUpdateVariantRules' operation
    
    :var index: Specifies the corresponding variant rule input index.
    :var ruleObject: An object that carries variant configuration criteria.
    :var variantExpression: The list of  variant expressions that represent the variant configuration criteria
    associated with this object.
    """
    index: int = 0
    ruleObject: POM_object = None
    variantExpression: List[ConfigExpression] = ()


@dataclass
class Violation(TcBaseObj):
    """
    Represents a violated rule along with the message and severity associated with rule violation.
    
    :var message: The message to display if this rule is violated.
    :var severity: The severity associated with the message. Valid values for this parameter are defined by the
    ListOfValues "Cfg0ConstraintSeverity".
    :var violatedRule: The rule that was found to be violated.
    """
    message: str = ''
    severity: str = ''
    violatedRule: ConfigExpression = None


@dataclass
class BusinessObjectConfigExpression(TcBaseObj):
    """
    Relates a business object (e.g. Cpd0DesignElement, VariantRule, or Cfg0ExcludeRule) to a list of configuration
    expressions.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var targetObject: The business object (e.g. Cpd0DesignElement, VariantRule, or Cfg0ExcludeRule), to which the
    configuration expressions specified in parameter 'expressions' are linked.
    :var expressions: The list of expressions related to the business object specified in parameter 'targetObject'. The
    list of suported expression categories varies from business object type to business object type. The expression
    category is specified with the expressionType parameter in each expression.
    """
    clientId: str = ''
    targetObject: POM_object = None
    expressions: List[ConfigExpression] = ()


@dataclass
class ConfigExpression(TcBaseObj):
    """
    Defines a Boolean expression to be used for configuration purposes.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var expressionType: Specifies the expression category and the intent of this expression. 
    
    The valid values are as follows:
    - 25 - Applicability
    - 28 - Inclusive Constraint
    - 29 - Exclusive Constraint
    - 33 - Product Model
    - 37 - Default Rule
    
    
    :var formula: The persistent formula string in Teamcenter syntax (see documentation for property
    ''cfg0SubjectCondition'' or BOMLine property ''bl_formula'').
    :var subExpressions: Represents a list of sub-expressions in a Boolean expression. The sub-expressions in this list
    are combined with logical operator 'OR'.
    """
    clientId: str = ''
    expressionType: int = 0
    formula: str = ''
    subExpressions: List[ConfigSubExpression] = ()


@dataclass
class ConfigExpressionDisplayStringInput(TcBaseObj):
    """
    An input structure defining the expressions for which the display format is requested.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var expressions: The expressions for which the display format is requested.
    """
    clientId: str = ''
    expressions: List[ConfigExpression] = ()


@dataclass
class ConfigExpressionDisplayStringOutput(TcBaseObj):
    """
    Output structure returned by operation getVariantExpressionDisplayStrings. It contains the list of display strings
    for the input expressions.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var expressionStrings: The display strings for each input expression.
    """
    clientId: str = ''
    expressionStrings: List[str] = ()


@dataclass
class ConfigExpressionGroup(TcBaseObj):
    """
    This structure groups 'ConfigExpressionTerm' structures. A mixture of positive and negated expression terms in the
    same expression group is not allowed.  Positive terms within an expression group are combined with operator 'OR',
    e.g. Color=Red | Color=Green. Negated terms within an expression group are combined with operator 'AND', e.g.
    Color!=Red & Color!=Green
    
    :var groupName: The name for the group containing the expression terms. If the group name is provided as FALSE with
    empty term expressions then group expression would be considered as FALSE. If the group name is provided as TRUE
    with empty term expressions then group expression would be considered as TRUE.
    :var terms: List of ConfigExpressionTerm structures
    """
    groupName: str = ''
    terms: List[ConfigExpressionTerm] = ()


@dataclass
class ConfigExpressionTerm(TcBaseObj):
    """
    Defines an elemental expression literal for a single feature, or feature range expression for a given family.
    
    :var familyNamespace: The option family namespace. This parameter is mutually exclusive with parameter 'value'.
    When this structure is used in a request, this parameter is ignored if a valid family business object is specified
    in parameter 'family'
    :var familyID: The ID of the family in the term. This parameter is mutually exclusive with parameter 'value'. When
    this structure is used in a request, this parameter is ignored if a valid family business object is specified in
    parameter 'family'
    :var operatorCode: The operator code to use for this expression terms.
    The Valid values are as follows:
    - 5 - Equals
    - 6 - Not Equals
    - 10 - And
    - 11 - OR
    
    
    :var valueText: The ID of the feature object or the feature to use for a free form family. This parameter is
    mutually exclusive with parameters 'family' and 'rangeExpressions'. It is recommended to author variant conditions
    involving free form features by using inequation operators in parameter 'rangeExpressions'. Equality condition for
    free form features should only be used in variant configuration criteria. In that case parameter 'valueText' may be
    used to specify a free form feature. When this structure is used in a request, this parameter is ignored if a valid
    feature business object is specified in parameter 'value'.
    :var rangeExpressions: A list of elemental expression literals describing a value range for a given family. This
    parameter requires a family to be specified with parameters 'familyNamespace' and 'familyID', or 'family'. This
    parameter is mutually exclusive with parameter 'value'.
    :var family: Specifies family object to be used in a free form family expression term, or a feature range
    expression. This parameter is mutually exclusive with parameter 'value'.
    :var value: Specifies feature object to be used in an elemental expression literal. This parameter is mutually
    exclusive with parameters 'family' and 'rangeExpressions'.
    """
    familyNamespace: str = ''
    familyID: str = ''
    operatorCode: int = 0
    valueText: str = ''
    rangeExpressions: List[RangeExpression] = ()
    family: Cfg0AbsFamily = None
    value: Cfg0AbsValue = None


@dataclass
class ConfigSubExpression(TcBaseObj):
    """
    This structure groups 'ConfigExpressionGroup' structures which are joined together by "AND" operator.
    
    :var expressionGroups: Represents a list of clauses in a Boolean expression. The clauses in this list are combined
    with logical operator AND.
    """
    expressionGroups: List[ConfigExpressionGroup] = ()


@dataclass
class ConfiguratorConstraint(TcBaseObj):
    """
    A SOA structure representation of Teamcenter::ConfiguratorConstraint. The data members of this SOA structure
    correspond the Teamcenter runtime objects in namespace Teamcenter::Math. For example, a
    ConfigurationManagement::VaraintOption corresponds to a Math::VariantOption.
    
    :var modelDesignator: Text representation of model condition defined on a rule.
    :var applicability: Text representation of applicability condition defined on a rule.
    :var restriction: Text representation of subject condition defined on a rule.
    :var comment: User entered message on the rule. This string is a localizable string which is displayed to the user
    when any rule execution fails.
    :var modelOptions: A list of Variation families that are associated with the model condition of a rule.
    :var applicabilityOptions: A collection of Variation families that are associated with the applicability condition
    of a rule.
    :var restrictionOptions: A collection of Variation families that are associated with the subject condition of a
    rule.
    :var ruleOptions: A unique collection of all the families, model, applicability and subject, that are contained
    within this constraint object.
    :var constraintType: The type of constraint.
    Valid values for this parameter are as follow:
    - IncludeConstraint: A constraint rule that is used to validate and possibly modify the configuration criteria.
    Validation fails whenever adding the condition of this constraint rule with operator AND would result in
    unsatisfiable configuration criteria. If the scope of the input variant criteria exceeds the scope of the
    constraint rule condition, the scope is reduced to stay within the bounds of the constraint rule scope by ANDing
    the rule condition to the configuration criteria.
    
    
    
    - ExcludeConstraint: A constraint rule that is used to validate configuration criteria without modifying it.
    Validation fails when the configuration criteria satisfy the condition of this constraint rule.
    
    
    :var constraintSeverity: The severity associated with the message. Valid values for this parameter are defined by
    the ListOfValues "Cfg0ConstraintSeverity".
    """
    modelDesignator: Expression = None
    applicability: Expression = None
    restriction: Expression = None
    comment: str = ''
    modelOptions: List[VariantOption] = ()
    applicabilityOptions: List[VariantOption] = ()
    restrictionOptions: List[VariantOption] = ()
    ruleOptions: List[VariantOption] = ()
    constraintType: str = ''
    constraintSeverity: str = ''


@dataclass
class CreateInput(TcBaseObj):
    """
    This structure captures the inputs required for creation of a business object. It is a nested structure containing
    the 'CreateInput'(s) for any secondary(compounded) objects that might be created (e.g Item also creates
    ItemRevision and ItemMaster Form, etc.). This structure should only be used for purposes specific to this Service
    Interface, e.g. when creating transient VariantRules. Generally business objects should be created with the
    DataManagement Service Interface.
    
    :var boName: Business object type name
    :var stringProps: Map of string property names to values ('string, string')
    :var doubleArrayProps: Map of double array property names to values map( string, double)
    :var floatProps: Map of float property names to values (string, bool)
    :var floatArrayProps: Map of float array property names to values map( string, float)
    :var objectProps: Map of BusinessObject property names to values ('string, BusinessObjec't)
    :var objectArrayProps: Map of BusinessObject array property names to values ('string, vector<BusinessObject>')
    :var compoundCreateInput: Map of reference or relation property name to secondary 'CreateInput' objects '(string,
    vector<CreateInput>')
    :var stringArrayProps: Map of string array property names to values ('string, vector<string>')
    :var boolProps: Map of boolean property names to values (string, bool)
    :var boolArrayProps: Map of boolean array property names to values ('string, vector<bool>')
    :var dateProps: Map of DateTime property names to values ('string, DateTime')
    :var dateArrayProps: Map of DateTime array property names to values ('string, vector<DateTime>')
    :var intProps: Map of integer property names to values (string, int)
    :var intArrayProps: Map of ineteger array property names to values map( string, int)
    :var doubleProps: Map of double property names to values (string, double)
    """
    boName: str = ''
    stringProps: StringMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    objectProps: TagMap = None
    objectArrayProps: TagVectorMap = None
    compoundCreateInput: CreateInputMap = None
    stringArrayProps: StringVectorMap = None
    boolProps: BoolMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    doubleProps: DoubleMap = None


@dataclass
class CreateUpdateVariantRulesResponse(TcBaseObj):
    """
    Response structure for the 'createUpdateVariantRules' operation.
    
    :var ruleOutputs: List of variant rule output structures
    :var serviceData: Service Data
    """
    ruleOutputs: List[VariantRuleOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DeclaredFamily(TcBaseObj):
    """
    A SOA structure representation of 'Teamcenter'::'ConfigurationFamilyFactory'::'DeclaredFamily'.
    
    :var family: SOA representation of Math::VaraintOption that defines an family.
    :var valueRange: A SOA expression representation of valueRange, Math::BooleanExpression, present in
    Teamcenter::ConfigurationFamilyFactory::DeclaredFamily.
    :var values: The map of values in pairs of (string, string) for family and feature.
    :var variantMode: A SOA representation of variantMode data member in
    Teamcenter::ConfigurationFamilyFactory::DeclaredFamily.
    """
    family: VariantOption = None
    valueRange: Expression = None
    values: StringVariantOptionMap = None
    variantMode: int = 0


@dataclass
class Expression(TcBaseObj):
    """
    A SOA structure that represents the 'Math'::'BooleanExpression' as a text. The collection of these text expressions
    in the vector is the output returned from 'Cfg'::'expressionsAsStrings'(). The string representation can be
    converted back into 'Math'::'BooleanExpression' by invoking 'Cfg'::'getExpressions'().
    
    :var id: A unique ID associated with this expression structure. Note: Currently this attribute is unused. The main
    intention of using this id is to be able to reduce the data over the wire by serializing only one Expression
    structure and leverage this ID wherever it is used.
    :var asTextExpression: Text representation of Math::BooleanExpression returned from Cfg::expressionsAsStrings().
    """
    id: int = 0
    asTextExpression: List[str] = ()


@dataclass
class ExpressionGroupInput(TcBaseObj):
    """
    Aggregates multiple search criteria into complex search expressions.
    
    :var clientId: Identifies this structure within the scope of an operation. The identifier allows this structure to
    be referenced from other search definition input strructures in the same operation, e.g. this structure could be
    referenced from within an enclosing ExpressionGroupInput structure.
    :var searchOperator: The set operator to use when combining the results from each search definition in this group.
    Possible values are:
    - "UnionAll"
    - "UnionUnique"
    - "Intersection"
    - "Difference"
    
    
    Use "UnionAll" if search results are not expected to contain duplicates, or if duplicates are not a concern. The
    search expressions in this group are combined in the sequence in which they appear in the group.
    
    "UnionAll" and "UnionUnique" combine the results of two search definitions into a set that contains all of the
    objects that are returned from either the left or the right search definition operand. "UnionAll" does not remove
    duplicates and is therefore often faster.
    
    "Intersection" combines the results of two search definitions into a set that contains all of the objects that are
    returned from the left as well as the right search definition operand.
    
    "Difference" combines the results of two search definitions into a set that contains all of the objects that are
    returned from the left but not from the right search definition operand.
    
    Search definitions constructed from a 'ResultTypeInput' are always combined with "Intersection".
    
    Exclude search definitions constructed from a 'ObjectReferenceInput' are always combined with "Difference".
    
    Include search definitions constructed from a 'ObjectReferenceInput' are combined with "UnionAll" if the set
    operator for the group is "UnionAll", otherwise with "UnionUnique".
    
    For example, a "UnionAll" group that combines the following search definitions:
    - "savedQuery"
    - "includeObjects"
    - "resultTypes"
    - "excludeObjects"
    
    
    would return:
    $savedQuery UnionAll $includeObjects Intersection $resultTypes Difference $excludeObjects.
    :var expressionIds: The list of client Ids of the search expressions to combine in this group. Any client ID that
    is assigned to a search definition input structure within the same 'SearchExpressionInput' can be used, including
    other 'ExpressionGroupInput' client ID values. For example you could specify two client ID values that are given in
    the "'savedQueries'" parameter of the SearchExpressionInput that also contains this 'ExpressionGroupInput'
    structure if you want to intersect or union the results from these two searches. Or you could combine one of the
    client ID values in parameter "'savedQueries'" with one of the ID values in parameter "'excludeObjects'" if you
    want to explicitly remove some objects from the search results of the saved query. This 'ExpressionGroupInput' then
    forms a new search criterion as a combination of the results from the search definitions that are referenced in
    this list.
    :var settings: Search pragmata, which describe how the server should should process the input. 
    Note: Support for search pragmata is not implemented in Teamcenter 10.1.2.
    """
    clientId: str = ''
    searchOperator: SearchGroupOperator = None
    expressionIds: List[str] = ()
    settings: SettingsMap = None


@dataclass
class ExpressionJoinCondition(TcBaseObj):
    """
    Defines the join condition for the results of two searches.
    An 'ExpressionJoinCondition' with values
    - left=2
    - operatorCode=1500
    - right=3
    
    
    defines a join condition between values in the 2nd column of the left and the 3rd column of the right search using
    the EQUAL comparison operator.
    
    :var left: The index of the column in the left search in this join.
    :var operatorCode: The operator to use in this join condition when matching the columns in the left and right
    search. Supported operators are:
    - 15000 (POM_enquiry_equal)
    
    
    :var right: The index of the column in the right search in this join.
    """
    left: int = 0
    operatorCode: int = 0
    right: int = 0


@dataclass
class ExpressionJoinInput(TcBaseObj):
    """
    Combines multiple search criteria into a single search expression returning a result table with multiple columns.
    
    :var clientId: Identifies this structure within the scope of an operation. The identifier allows this structure to
    be referenced from other search definition input strructures in the same operation, e.g. this structure could be
    referenced from within an enclosing ExpressionGroupInput structure.
    :var searchOperator: The join operator to use when combining the results from each search definition in this join.
    :var expressionIds: The list of client Ids of the search expressions to combine in this join. Any client ID that is
    assigned to a search definition input structure within the same 'SearchExpressionInput' can be used, including
    other 'ExpressionJoinInput' client ID values. For example you could specify two client ID values that are given in
    the "'savedQueries'" parameter of the 'SearchExpressionInput' that also contains this 'ExpressionGroupInput'
    structure if you want to join the results from these two searches. Or you could combine one of the client ID values
    in parameter "savedQueries" with one of the ID values in parameter "'includeObjects'" if you want to explicitly
    join some objects from the search results of the saved query. This 'ExpressionJoinInput' then forms a new search
    criterion as a combination of the results from the search definitions that are referenced in this list.
    :var settings: Search pragmata, which describe how the server should should process the input.
    Note: Support for search pragmata is not implemented in Teamcenter 10.1.3.
    :var joinConditions: Defines the join condition to use when joining the 1st search in parameter "'expressionIds'"
    with the other searches parameter "'expressionIds'".
    Teamcenter 10.1.3 only support one single join condition.
    """
    clientId: str = ''
    searchOperator: SearchJoinOperator = None
    expressionIds: List[str] = ()
    settings: SettingsMap = None
    joinConditions: List[ExpressionJoinCondition] = ()


@dataclass
class FamilyFilters(TcBaseObj):
    """
    A filter filtering rules for all features belonging to all families specified in this structure.
    
    :var familyIDs: List of family IDs ( 'cfg0object_id' ). If specified all families with matching criteria will be
    searched and values belongs to searched families will be used to filter rules. This parameter is mutually exclusive
    of 'families'.
    :var families: List of families. If specified values belongs to all families will be used to filter rules. This
    parameter is mutually exclusive of 'familyIds'.
    """
    familyIDs: List[str] = ()
    families: List[Cfg0AbsOptionFamily] = ()


@dataclass
class FilterCriteria(TcBaseObj):
    """
    Criteria that allows filtering rule objects. The filter criteria can be given in the following ways:
    - If 'groupFilters' is populated, then the system shall find all groups within the current context that match the
    filter and find rules that reference all features within families of the group.
    - If 'familyFilters' is populated, then the system shall find all families within the current context that match
    the filter and find rules that reference all features within the families.
    - If 'valueFilters' is populated, then the system shall find all features within the current context that match the
    filter and find rules that reference the features.
    - Criteria within multiple fields are intersected.
    
    
    
    :var groupFilters: Filter for filtering based on all features belonging to families in specified groups.
    :var familyFilters: Filter for filtering based on all features belonging to families specified.
    :var valueFilters: Filter for filtering based on all features specified.
    :var includeUnreferencedRules: This parameter determines whether or not rules that do not have any variant
    expressions set on them be returned or not.
    
    If this parameter is set to 'true', all such rules would be returned. If this parameter is set to 'false', such
    rules are not returned.
    """
    groupFilters: GroupFilters = None
    familyFilters: FamilyFilters = None
    valueFilters: ValueFilters = None
    includeUnreferencedRules: bool = False


@dataclass
class GetDisplayStringResponse(TcBaseObj):
    """
    Response structure for 'getVariantExpressionDisplayStrings' operation
    
    :var displayStrings: List of display strings for the input expressions
    :var serviceData: Service Data
    """
    displayStrings: List[ConfigExpressionDisplayStringOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetFamilyGroupResponse(TcBaseObj):
    """
    Response structure to return allocated groups and their corresponding allocation objects.
    
    :var groupAndAllocation: The Map of ( Cfg0AbsFamilyGroup, Cfg0AbsAssociation) having a family group to the
    corresponding allocation objects for the product item.
    :var serviceData: Teamcenter Service Data.
    """
    groupAndAllocation: GroupAllocationMap = None
    serviceData: ServiceData = None


@dataclass
class GetOptionFamiliesResponse(TcBaseObj):
    """
    A list of structures consisting of family groups and families that are attached to the respective family groups.
    
    :var groupAndFamilies: List of structure with the family groups and families associated with them. A
    'GroupAndFamilies' structure describes an instance of family group and the families which are associated with it.
    :var serviceData: Teamcenter service data.
    """
    groupAndFamilies: List[GroupAndFamilies] = ()
    serviceData: ServiceData = None


@dataclass
class GetOptionValuesResponse(TcBaseObj):
    """
    A list of structures having families and features of the respective families.
    
    :var valueAndAssocations: This is the map of feature objects and their corresponding association objects with the
    product item or the product model. The product item and the product model is associated with a features by an
    allocation object and an availability object, respectively.
    :var serviceData: Teamcenter service data.
    """
    valueAndAssocations: ValueAssociationMap = None
    serviceData: ServiceData = None


@dataclass
class GetProductDefaultsResponse(TcBaseObj):
    """
    The response structure for operation 'GetProductDefaults'
    
    :var expressionsWithDefaults: Contains the output expression that result from aplying default rules to the input
    expressions.
    :var serviceData: The service data to return any additional information.
    """
    expressionsWithDefaults: List[BusinessObjectConfigExpression] = ()
    serviceData: ServiceData = None


@dataclass
class GetVariantCacheInfoResponse(TcBaseObj):
    """
    Revision configured variant data for the given product 'context' item revision.
    
    :var variantInfo: Contains revision configured variant data such as variant features, families, and rules.
    :var serviceData: Service data of teamcenter.
    """
    variantInfo: VariantInfo = None
    serviceData: ServiceData = None


@dataclass
class GroupAndFamilies(TcBaseObj):
    """
    The structure with the family group and its associated families.
    
    :var familyGroup: A family group object.
    :var familyAndAssociation: The Map of ( Cfg0AbsFamilyGroup, Cfg0AbsAssociation)  having a family group to the
    corresponding allocation objects for the product item.
    """
    familyGroup: Cfg0AbsFamilyGroup = None
    familyAndAssociation: FamilyAssociationMap = None


@dataclass
class GroupFilters(TcBaseObj):
    """
    A filter structure specifying filtering rules for all option values belonging to all families to group.
    
    :var groupIDs: List of group IDs ( 'cfg0object_id' ). If specified all groups with matching criteria will be
    searched and values belongs to all families in searched groups will be used to filter rules. This parameter is
    mutually exclusive of 'groups'.
    :var groups: List of groups. If specified values belongs to all families in groups will be used to filter rules.
    This parameter is mutually exclusive of 'groupIds'.
    """
    groupIDs: List[str] = ()
    groups: List[Cfg0AbsFamilyGroup] = ()


@dataclass
class ObjectReferenceInput(TcBaseObj):
    """
    Represents a list of object to by explicitly included in, or explicitly excluded from, the search results.
    
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var objects: Reference to a list of POM_Object objects used to create the include or exclude search expression.
    :var resultColumns: A list of additional columns to return in the search result. This search definition uses a
    predefined result table column for attribute "puid" at index 0. This parameter can be used to define additional
    columns to be returned in the result table for this search table.
    """
    clientId: str = ''
    objects: List[POM_object] = ()
    resultColumns: List[SearchResultTableColumn] = ()


@dataclass
class ProductDefaultsInput(TcBaseObj):
    """
    Input structure for the 'getProductDefaults' operation
    
    :var applyDefaults: Indicates whether or not to apply defaults. The parameter value is a bit pattern specifying one
    or more of the following: 
    0 - indicates that defaults should not be applied 
    1 - will enable application of defaults 
    2 - ignore all derived defaults and only process fixed defaults 
    4 - skip derived defaults whose applicability or product model condition is not fully and explicitly satisfied. For
    example, a default rule 'set W=10 if H=10' is skipped for the input expression 'L=10' because 'L=10' also includes
    configurations that don't satisfy the applicabliity condition, such as 'L=10 & H=20'.
    8 - force partially satisfied derived defaults. For example, applying a default rule 'set W=10 if H=10' to an input
    expression 'L=10' results in 'W=10 & H=10 & L=10'
    :var expressionsToUpdate: The expressions for which defaults are requested to be applied.
    :var families: This is an optional parameter. If specified, the server returns defaults only for these families. If
    null, the server returns defaults for all families.
    :var context: An Cfg0ConfiguratorPerspective object to provide input data which includes product item(s), product
    model(s). It may also include configuration information e.g. a revision rule and/or a variant rule.
    """
    applyDefaults: int = 0
    expressionsToUpdate: List[BusinessObjectConfigExpression] = ()
    families: List[Cfg0AbsFamily] = ()
    context: Cfg0ConfiguratorPerspective = None


@dataclass
class RangeExpression(TcBaseObj):
    """
    An elemental expression literal that is used in a value range expression for a given family.
    Supported operators are (see ):
     - PS_variant_operator_not_equal
     - PS_variant_operator_gt
     - PS_variant_operator_lt
     - PS_variant_operator_gt_eq
     - PS_variant_operator_lt_eq
    Operator PS_variant_operator_is_equal is not supported (use ConfigExpressionTerm structures for equality literals).
    
    :var operatorCode: Operator Code
    :var valueText: The ID of the feature object or the feature to use for a free form family.
    """
    operatorCode: int = 0
    valueText: str = ''


@dataclass
class ResultTypeInput(TcBaseObj):
    """
    Defines a search result filter based on a list of business object types. The search results of the enclosing
    ConfigExpressionGroup are limited to objects that match one of the given businesses object types, or one of their
    sub types. Search definitions enclosed in a ConfigExpressionGroup that are constructed from a ResultTypeInput are
    combined with operation "Intersection". ResultTypeInput can also be used to perform a SELECT operation on a list of
    business object type, or their subtypes tables. As the result could sometimes be very large, user can filter the
    result by providing a where clause condition. You can refer to the description of SearchPredicateParameters.
    
    :var clientId: Identifies this structure within the scope of an operation. The identifier allows this structure to
    be referenced from other search definition input strructures in the same operation, e.g. this structure could be
    referenced from within an enclosing ExpressionGroupInput structure.
    :var searchResultObjectTypes: List of the search result type names. The search will filter out objects that don't
    match any of the business object types in this list, or their sub types.
    :var resultColumns: A list of additional columns to return in the search result. This search definition uses a
    predefined result table column for attribute "puid" at index 0. This parameter can be used to define additional
    columns to be returned in the result table for this search table.
    :var predicateClauseParameters: A list of WHERE clause parameters to filter this ResultType search definition.
    Adding a WHERE clause can significantly improve the performance, especially when the corresponding database tables
    are large.
    """
    clientId: str = ''
    searchResultObjectTypes: List[str] = ()
    resultColumns: List[SearchResultTableColumn] = ()
    predicateClauseParameters: List[SearchPredicateParameters] = ()


@dataclass
class SavedQueryExpressionInput(TcBaseObj):
    """
    Defines a search based on a saved query that was created in the Teamcenter QueryBuilder application.
    
    :var clientId: Identifies this structure within the scope of an operation. The identifier allows this structure to
    be referenced from other search definition input strructures in the same operation, e.g. this structure could be
    referenced from within an enclosing ExpressionGroupInput structure.
    :var entries: The list of entry names specified in the saved query for which values are specified in parameter
    'values'.
    :var values: The list of values for the specified entries. Wild card characters are supported as documented for the
    Teamcenter QueryBuilder application.
    :var savedQuery: A saved query object that was created in the Teamcenter QueryBuilder application (or its
    corresponding APIs).
    :var resultColumns: A list of additional columns to return in the search result. This search definition uses a
    predefined result table column for attribute "puid" at index 0. This parameter can be used to define additional
    columns to be returned in the result table for this search table.
    """
    clientId: str = ''
    entries: List[str] = ()
    values: List[str] = ()
    savedQuery: ImanQuery = None
    resultColumns: List[SearchResultTableColumn] = ()


@dataclass
class SearchExpressionInput(TcBaseObj):
    """
    Describes the search criteria for the search operation
    
    :var traversals: Input structures to define traversal expressions. Teamcenter 10.1.2 does not support traversal
    expressions.
    :var excludeObjects: A list of search definitions for objects that explicitly excluded from the search results.
    Exclude lists are not subject to context filtering according to the product context items and product models in the
    'Cfg0ConfiguratorPerspective'. Exclude search definitions are always combined with operator "Difference", even if
    they are contained in a "UnionAll" group search definition.
    :var includeObjects: A list of search definitions for objects that explicitly included to the search results.
    Include lists are subject to context filtering according to the product context items and product models in the
    Cfg0ConfiguratorPerspective. Include search definitions that are contained in a "UnionAll" group search definition
    are combined with operator "UnionAll", all other include search definitions are combined with operator
    "UnionUnique" even if they are contained in a "Difference" group search definition.
    :var savedQueries: Input structures to construct search definitions based on saved queries. Each saved query search
    definitions references a saved query object along with a list of entries and values. Multiple values for a given
    entry need to be separated according to preference 'WSOM_find_list_separator'. The values support the standard
    Teamcenter saved query wildcards characters.
    :var expressionGroups: Input structures for constructing group search definitions. The input for a group search
    definition may contain references to other group search definitions for as long as the referencee precedes the
    referencer in this list. Group search definitions can be used to aggregate search criteria for complex searches.
    :var expressionJoins: Input structures for constructing join search definitions. Join search definitions can be
    used to combine multiple search criteria into a single search expression returning a result table with multiple
    columns.
    The input for a join search definition may contain references to other join search definitions for as long as the
    referencee precedes the referencer in this list.
    :var resultTypes: Input structure to define result type filter definitions. When applied to a search the result
    will contain only objects that match one of the business object type names listed in the applied 'ResultTypeInput',
    or one of their sub types.
    """
    traversals: List[TraversalExpressionInput] = ()
    excludeObjects: List[ObjectReferenceInput] = ()
    includeObjects: List[ObjectReferenceInput] = ()
    savedQueries: List[SavedQueryExpressionInput] = ()
    expressionGroups: List[ExpressionGroupInput] = ()
    expressionJoins: List[ExpressionJoinInput] = ()
    resultTypes: List[ResultTypeInput] = ()


@dataclass
class SearchOptions(TcBaseObj):
    """
    Search options for a given search, such as load count and sort criteria.
    
    :var defaultLoadCount: The number of objects to return in the initial batch of search results. The rest of them
    could be retrieved by calling fetchNextSearchResults. A defaultLoadCount of zero will return all the results found.
    :var sortAttributes: A list of attribute names of the class being searched based on which the results are sorted.
    The position of an attribute in this list defines its precedence ranking with respect to sorting. The result is
    first sorted based on the first attribute in this list.
    :var sortOrder: order in which results are sorted
    """
    defaultLoadCount: int = 0
    sortAttributes: List[str] = ()
    sortOrder: List[SortOperator] = ()


@dataclass
class SearchPredicateParameters(TcBaseObj):
    """
    A list of WHERE clause parameters to filter ResultType search definition. Adding a WHERE clause can significantly
    improve the performance, especially when the corresponding database tables are large.
    
    :var attribute: The name of the Column on which the where condition is to be applied.
    :var valueObjects: The list of Input value objects to be evaluated in the results, this forms the predicate
    condition.
    :var predicateOperator: The Predicate condition operator. The supported operators for this parameter are:
    - 15000 (POM_enquiry_equal)
    - 15005 (POM_enquiry_not_equal)
    - 16501 (POM_enquiry_in)
    - 16502 (POM_enquiry_not_in)
    
    
    The semantics for these values follow the definitions in pom_tokens.h
    """
    attribute: str = ''
    valueObjects: List[POM_object] = ()
    predicateOperator: int = 0


@dataclass
class SearchRecipe(TcBaseObj):
    """
    The recipe for executing a search.
    
    :var configuratorPerspective: Defines the scope of the search. Results are filtered based on the parameters defined
    by the perspective, e.g. revision rule, variant rule, product item(s), and product model(s).
    :var settings: Processing directives, for example parameters to pass on to the configurator for evaluation of
    variant configuration constraint rules. 
    
    The settings which are supported  are:
    
    Cfg0PerspectiveColumnsToReturn  - List of strings used to specify the column indices of the result table that
    should be returned back to the client. For example: if the list  contains { "0", "2" }, only columns at 0th and 2nd
    index will be returned back to the client. If no value is specified, all columns will be returned.
    :var searchExpression: Search criteria to be evaluated
    :var rootExpressionInputClientId: The 'clientId' value of the  root search definition structure . The search
    definitions that are contained in parameter "'searchExpression'" are the individual building blocks that aggregate
    into a single query. The "'rootExpressionInputClientId'" parameter identifies which of the search criteria
    definitions in "'searchExpression'" represents the composition of all individual search criteria. For example if
    "'searchExpression'" contained a 'savedQuery' "Q", an 'includeObject' definition "I", and an 'expressionGroup' "G"
    which combines "Q" and "I", then the 'rootExpressionInputClientId' would be "G".
    :var variantCriteria: Input Object with Variant Expression to be used for filtering the search result. 
    This filter is having 'AND' relation with the searchExpression and it is applied applied on top of
    searchExpression. 
     
    The expresion used for filtering is based on the 'opcode' passed as input in settings.
    If no Variant Expression is found for the 'opcode' passed as input or the input object is not Variant Configurable
    then following validation error is thrown:
    79008 - The search criteria provided in input is invalid.
    """
    configuratorPerspective: Cfg0ConfiguratorPerspective = None
    settings: SettingsMap = None
    searchExpression: SearchExpressionInput = None
    rootExpressionInputClientId: str = ''
    variantCriteria: BusinessObject = None


@dataclass
class SearchResponse(TcBaseObj):
    """
    Response SOA Structure for search results
    
    :var objectsDone: "Deprecated, use rowsDone instead"
    An integer value specifying the number of objects returned so far for executeSearch and fetchNextSearchResults
    operations.
    :var rowsDone: An integer value specifying the number of rows returned so far for executeSearch and
    fetchNextSearchResults operations.
    :var estimatedObjectsLeft: "Deprecated, use estimatedRowsLeft instead"
    An integer value specifying the estimated number of objects that still are potentially results of this search.
    :var estimatedRowsLeft: An integer value specifying the estimated number of rows that still are potentially results
    of this search.
    :var foundObjects: The list of objects returned by the executeSearch or fetchNextSearchResults operation.
    :var searchResultTable: A table of rows returned by the executeSearch or fetchNextSearchResults operation.
    :var searchCursor: Search cursor object that tracks the search results. This object is used to get the next set of
    results for this executeSearch operation.
    :var serviceData: Service Data for any error information. Typically this will contain errors about any malformed
    search recipes. 
    Following are some of the error codes that may be populated as partial errors in the ServiceData object(To be added
    later)
    """
    objectsDone: int = 0
    rowsDone: int = 0
    estimatedObjectsLeft: int = 0
    estimatedRowsLeft: int = 0
    foundObjects: List[POM_object] = ()
    searchResultTable: SearchResultTable = None
    searchCursor: Cfg0SearchCursor = None
    serviceData: ServiceData = None


@dataclass
class SearchResultTable(TcBaseObj):
    """
    Represents a set of search result records. Each result column is return in the member that corresponds to the data
    type for this column. For example, a search that produced a result table comprising an object column, a string
    column, and another object column will return the two object columns in member "'objectColumns'" as
    'objectColumns''["column__''0''"]' and 'objectColumns''["column__''2''"]', while the string value column will be
    returned in member "'stringColumns'" as 'stringColumns''["column__''1''"]'.
    
    :var objectColumns: A map tying a column header to a vector of objects. The column header corresponds to the
    "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the 'SearchExpressionInput' that
    produced this result column. If no 'alias' name was assigned a system generated column header like "column__0" will
    be used.
    :var stringColumns: A map tying a column header to a vector of string values. The column header corresponds to the
    "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the' ''SearchExpressionInput'
    that produced this result column. If no alias name was assigned a system generated column header like "column__0"
    will be used.
    :var intColumns: A map tying a column header to a vector of integer values. The column header corresponds to the
    "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the 'SearchExpressionInput' that
    produced this result column. If no alias name was assigned a system generated column header like "column__0" will
    be used.
    :var doubleColumns: A map tying a column header to a vector of double precision floating point values. The column
    header corresponds to the "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the
    'SearchExpressionInput' that produced this result column. If no alias name was assigned a system generated column
    header like "column__0" will be used.
    :var logicalColumns: A map tying a column header to a vector of Boolean values. The column header corresponds to
    the "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the 'SearchExpressionInput'
    that produced this result column. If no alias name was assigned a system generated column header like "column__0"
    will be used.
    :var dateColumns: A map tying a column header to a vector of date values. The column header corresponds to the
    "'alias'" field that was specified in the 'SearchResultTableColumn' definition in the 'SearchExpressionInput' that
    produced this result column. If no alias name was assigned a system generated column header like "column__0" will
    be used.
    """
    objectColumns: TagVectorMap = None
    stringColumns: StringVectorMap = None
    intColumns: IntVectorMap = None
    doubleColumns: DoubleVectorMap = None
    logicalColumns: BoolVectorMap = None
    dateColumns: DateVectorMap = None


@dataclass
class SearchResultTableColumn(TcBaseObj):
    """
    Defines a column to be returned in the search result table for this search definition. Some search definitions use
    a predefined column for attribute "puid" at index 0.
    
    :var index: The index of the column in the search result table. Some search definitions use a predefined column for
    attribute "puid" at index 0. For these search definitions the index should be >= 1.
    :var attribute: The name of the POM attribute to return in this column of the search result table.
    :var alias: The alias name of the search result table column. A NULL value indicates that no alias name is assigned.
    :var dataType: The data type of the search result table column. The value must match a POM type token between
    POM_MIN_type_token and POM_MAX_type_token defined in <pom/pom/pom_tokens.h>.
    """
    index: int = 0
    attribute: str = ''
    alias: str = ''
    dataType: int = 0


@dataclass
class TraversalExpressionInput(TcBaseObj):
    """
    Defines search criteria based on a TransferOptionSet along with a set of configuration parameters available for
    this TransferOptionSet. Search criteria defined by a 'TraversalExpressionInput' structure specify how the search
    results should be post processed. The given TransferOptionSet references a TransferMode and ClosureRule to traverse
    to objects that are related to the search results. TransferOptionSet search definitions are not supported in
    Teamcenter 10.1.2.
    
    :var clientID: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var transferOptionsSet: Defines the traversal to apply as a post process after executing a search query. The
    specified TransferOptionSet object contains to reference a TransferMode object along with the list of defined
    parameters for this transfer mode. The selected transfer mode contains a reference to the traversal rule
    (ClosureRule) to be used along with the definition of the output format, and the filters to apply.
    :var optionNames: The list of the names of the parameters for which a value is provided. The list of available
    names depends on the TransferOptionSet that is specified in parameter.
    :var optionValues: Specifies the values of the provided parameters. TransferOptionSet objects represent a
    parameterized transfer mode. The list of valid values depends on the specific transfer mode that is selected by the
    option set given in parameter transferOptionsSet.
    :var resultColumns: A list of additional columns to return in the search result. This search definition uses a
    predefined result table column for attribute "puid" at index 0. This parameter can be used to define additional
    columns to be returned in the result table for this search table.
    """
    clientID: str = ''
    transferOptionsSet: TransferOptionSet = None
    optionNames: List[str] = ()
    optionValues: List[str] = ()
    resultColumns: List[SearchResultTableColumn] = ()


@dataclass
class ValidateProductConfigInput(TcBaseObj):
    """
    Input structure for the 'validateProductConfiguration' SOA
    
    :var applyConstraints: Indicates whether or not to apply the constraints. The parameter value is a bit pattern
    specifying the requested operation. A parameter value of 25 reports warnings and informational messages in addition
    to errors : 
    Apply constraints | Report Warnings | Report Info = 1+8+16 = 25 
    0 - indicates constraint application is not required 
    1 - apply configuration constraints 
    2 - skip constraints if they only reference unset families 
    4 - skip constraints that don't reference all configExpression families 
    8 - report warnings in addition to errors 
    16 - also report informational messages 
    32 - continue Validation On Error 
    64 - treat constraints with warning severity as if they had error severity
    128 - skip all variability checks which could raise a "k_variant_criteria_outside" validation failure
    256 - skip the check against allocated variability which could raise a "k_variant_criteria_outside" validation
    failure
    1024 - skip criteria update based on validation rules ( e.g. Exclude rule ). If this flag is not set it will update
    the criteria after applying validation rules
    2048 - enable Availability Constraint evaluation
    :var expressionsToValidate: The expressions to be validated.
    :var context: A runtime object to provide input data which include product item(s), product model(s). It also
    include configuration information e.g. revision rule and or variant rule.
    """
    applyConstraints: int = 0
    expressionsToValidate: List[BusinessObjectConfigExpression] = ()
    context: Cfg0ConfiguratorPerspective = None


class SearchGroupOperator(Enum):
    """
    Set operator values to use when combining the results from each search definition in a 'ExpressionGroupInput'.
    - UnionAll     - Combines the results of two search definitions into a set that contains all of the objects that
    are returned from either the left or the right search definition operand. Duplicates are not removed from the
    combined set.
    - UnionUnique -    Combines the results of two search definitions into a set that contains a unique list of objects
    that includes objects returned from either the left or the right search definition operand. The combined set will
    not contain duplicates.
    - Intersection -    Combines the results of two search definitions into a set that contains all of the objects that
    are returned from the left as well as the right search definition operand.
    - Difference    - Combines the results of two search definitions into a set that contains all of the objects that
    are returned from the left but not from the right search definition operand.
    
    
    
    :var UnionAll: Combines the results of two search definitions into a set that contains all of the objects that are
    returned from either the left or the right search definition operand. Duplicates are not removed from the combined
    set.
    :var UnionUnique: Combines the results of two search definitions into a set that contains a unique list of objects
    that includes objects returned from either the left or the right search definition operand. The combined set will
    not contain duplicates.
    :var Intersection: Combines the results of two search definitions into a set that contains all of the objects that
    are returned from the left as well as the right search definition operand.
    :var Difference: Combines the results of two search definitions into a set that contains all of the objects that
    are returned from the left but not from the right search definition operand.
    """
    UnionAll = 'UnionAll'
    UnionUnique = 'UnionUnique'
    Intersection = 'Intersection'
    Difference = 'Difference'


class SearchJoinOperator(Enum):
    """
    Join operator values to use when combining the results from each search definition in a 'ExpressionJoinInput'.
    
    - InnerJoin - Combines the column values of two searches (A and B) based upon a join condition. The join compares
    each row of A with each row of B to find all pairs of rows which satisfy the join condition.
    - LeftJoin - The result of a left join for searches A and B always contains all records of the "left" search (A),
    even if the join condition does not find any matching record in the "right" search (B).
    - RightJoin - The result of a right join for searches A and B always contains all records of the "right" search
    (B), even if the join condition doesn't find any matching record in the "left" search (A).
    - FullJoin - The result of a full join for searches A and B always contains all records of both searches. Where
    records in A and B don't match, the result set will have NULL values for every column of the search that lacks a
    matching row. For those records that do match, a single row will be produced in the result set (containing fields
    populated from both searches).
    
    """
    InnerJoin = 'InnerJoin'
    LeftJoin = 'LeftJoin'
    RightJoin = 'RightJoin'
    FullJoin = 'FullJoin'


class SortOperator(Enum):
    """
    Sort operator values which decide the sort order for attributes in Search Options
    
    :var Ascending: Sort in ascending order
    :var Descending: Sort in descending order
    """
    Ascending = 'Ascending'
    Descending = 'Descending'


"""
Map of bool property names to values '(string, bool').
"""
BoolMap = Dict[str, bool]


"""
Map of bool array property names to values ('string, vector< bool >').
"""
BoolVectorMap = Dict[str, List[bool]]


"""
A map of family and its association with product item and model. Association object representing the family in the context of the product item or model.  If a product item has been specified, then an allocation object is included.  If a product model has been specified, then an availability object is included.
"""
FamilyAssociationMap = Dict[Cfg0AbsOptionFamily, List[Cfg0AbsAssociation]]


"""
A map comprising of fully qualified name of a family as a key and its contents - name, data type and other attributes.
"""
FamilyHash = Dict[str, DeclaredFamily]


"""
A map comprising of name of a family as a key and its contents - name, data type and other attributes.
"""
FamilyNameHash = Dict[str, List[DeclaredFamily]]


"""
Map of float property names to values ('string', 'float').
"""
FloatMap = Dict[str, float]


"""
Map of float array property names to values ('string', 'vector').
"""
FloatVectorMap = Dict[str, List[float]]


"""
A map which contains all allocation records for a group in product model.
"""
GroupAllocationMap = Dict[Cfg0AbsFamilyGroup, List[Cfg0AbsAllocation]]


"""
Map of ineteger property names to values ('string', 'int').
"""
IntMap = Dict[str, int]


"""
Map of integer array property names to values ('string', 'vector').
"""
IntVectorMap = Dict[str, List[int]]


"""
Maps processing directive names to their values, or value list
"""
SettingsMap = Dict[str, List[str]]


"""
Map of Strings
"""
StringMap = Dict[str, str]


"""
Map between family and variant option
"""
StringVariantOptionMap = Dict[str, VariantOption]


"""
Map of string array property names to values ('string, vector<string>').
"""
StringVectorMap = Dict[str, List[str]]


"""
Map of BusinessObject property names to values ('string, BusinessObject').
"""
TagMap = Dict[str, BusinessObject]


"""
Map of BusinessObject array property names to values ('string, vector<BusinessObject>').
"""
TagVectorMap = Dict[str, List[BusinessObject]]


"""
A map of values and its corresponding association with product item or model. Association object representing the feature in the context of the product item or model.  If a product item has been specified, then an allocation object is included.  If a product model has been specified, then an availability object is included.
"""
ValueAssociationMap = Dict[Cfg0AbsOptionValue, List[Cfg0AbsAssociation]]


"""
Map of reference or relation property name to secondary 'CreateInput' objects ('string, vector<CreateInput>').
"""
CreateInputMap = Dict[str, List[CreateInput]]


"""
Map of date property names to values ('string, date').
"""
DateMap = Dict[str, datetime]


"""
Map of DateTime array property names to values ('string, vector<DateTime>').
"""
DateVectorMap = Dict[str, List[datetime]]


"""
Map of  double property names to values (string, float).
"""
DoubleMap = Dict[str, float]


"""
Map of double array property names to values (string, vector).
"""
DoubleVectorMap = Dict[str, List[float]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Query._2008_06.SavedQuery import QueryInput
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeSavedQueries(cls, input: List[QueryInput]) -> SavedQueriesResponse:
        """
        Executes a set of saved queries of following type 
        QRY_RUN_BY_TC ( 0 ),QRY_RUN_BY_USER_QUERY ( 16 ), QRY_RUN_BY_KEYWORD_SEARCH ( 24 ). 
        
        The saved queries can be executed to yield results in 2 modes: 
        - Flat mode: In this traditional execution mode, only the first-level 
        objects (corresponding to the queried class) satisfying the query 
        are returned 
        - Hierarchical/Indented mode: This mode is only applicable for 
        saved queries that allow Hierarchical/Indented results. In this 
        execution mode, the first-level objects as well as any sub-level 
        objects satisfying the query criteria are returned. The hierarchy 
        level information is also returned so that result subsets can be 
        re-constructed using the resulting objects. 
        
        This service will retun the matched object UIDs. 
        
        After fetching UIDs, client need to call DataManagementService.loadObjects operation 
        to load objects by pages. 
        See the QueryInput and QueryResults data structures for 
        usage details.
        """
        return cls.execute_soa_method(
            method_name='executeSavedQueries',
            library='Query',
            service_date='2008_06',
            service_name='SavedQuery',
            params={'input': input},
            response_cls=SavedQueriesResponse,
        )



from tcsoa.gen.Cad._2008_03.services import StructureManagementService as imp0
from tcsoa.gen.Cad._2013_05.services import StructureManagementService as imp1
from tcsoa.gen.Cad._2007_01.services import StructureManagementService as imp2
from tcsoa.gen.Cad._2016_09.services import StructureManagementService as imp3
from tcsoa.gen.Cad._2019_06.services import StructureManagementService as imp4
from tcsoa.gen.Cad._2007_12.services import StructureManagementService as imp5
from tcsoa.gen.Cad._2008_06.services import StructureManagementService as imp6
from tcsoa.gen.Cad._2007_06.services import StructureManagementService as imp7
from tcsoa.gen.Cad._2016_03.services import DataManagementService as imp8
from tcsoa.gen.Cad._2007_01.services import DataManagementService as imp9
from tcsoa.gen.Cad._2007_12.services import DataManagementService as imp10
from tcsoa.gen.Cad._2008_03.services import DataManagementService as imp11
from tcsoa.gen.Cad._2008_06.services import DataManagementService as imp12
from tcsoa.gen.Cad._2010_09.services import DataManagementService as imp13
from tcsoa.gen.Cad._2012_09.services import DataManagementService as imp14
from tcsoa.gen.Cad._2009_04.services import StructureManagementService as imp15
from tcsoa.gen.Cad._2020_01.services import AppSessionManagementService as imp16
from tcsoa.gen.Cad._2007_09.services import StructureManagementService as imp17
from tcsoa.gen.Cad._2016_03.services import StructureManagementService as imp18
from tcsoa.gen.Cad._2011_06.services import DataManagementService as imp19
from tcsoa.gen.Cad._2014_10.services import DataManagementService as imp20
from tcsoa.gen.Cad._2018_06.services import StructureManagementService as imp21
from tcsoa.utils import TcService


class StructureManagementService(TcService):
    askChildPathBOMLines = imp0.askChildPathBOMLines
    askChildPathBOMLines2 = imp1.askChildPathBOMLines2
    closeBOMWindows = imp2.closeBOMWindows
    continueFindModelViews = imp3.continueFindModelViews
    continueReconcilePalette = imp3.continueReconcilePalette
    createBOMWindows = imp2.createBOMWindows
    createBOMWindows2 = imp1.createBOMWindows2
    createOrReConfigureBOMWindows = imp4.createOrReConfigureBOMWindows
    createOrUpdateAbsoluteStructure = imp2.createOrUpdateAbsoluteStructure
    createOrUpdateAbsoluteStructure2 = imp5.createOrUpdateAbsoluteStructure
    createOrUpdateAbsoluteStructure3 = imp6.createOrUpdateAbsoluteStructure
    createOrUpdateClassicOptions = imp7.createOrUpdateClassicOptions
    createOrUpdateRelativeStructure = imp2.createOrUpdateRelativeStructure
    createOrUpdateRelativeStructure2 = imp5.createOrUpdateRelativeStructure
    createOrUpdateRelativeStructure3 = imp6.createOrUpdateRelativeStructure
    createOrUpdateRelativeStructure4 = imp15.createOrUpdateRelativeStructure
    createOrUpdateRelativeStructure5 = imp1.createOrUpdateRelativeStructure
    createOrUpdateVariantConditions = imp7.createOrUpdateVariantConditions
    createOrUpdateVariantConditions2 = imp17.createOrUpdateVariantConditions2
    createVariantRules = imp6.createVariantRules
    deleteAssemblyArrangements = imp2.deleteAssemblyArrangements
    deleteAssemblyArrangements2 = imp5.deleteAssemblyArrangements
    deleteClassicOptions = imp7.deleteClassicOptions
    deleteRelativeStructure = imp2.deleteRelativeStructure
    deleteRelativeStructure2 = imp5.deleteRelativeStructure
    deleteRelativeStructure3 = imp6.deleteRelativeStructure
    deleteVariantConditions = imp7.deleteVariantConditions
    expandPSAllLevels = imp2.expandPSAllLevels
    expandPSAllLevels2 = imp6.expandPSAllLevels
    expandPSOneLevel = imp2.expandPSOneLevel
    expandPSOneLevel2 = imp6.expandPSOneLevel
    findModelViewsInStructure = imp18.findModelViewsInStructure
    getAbsoluteStructureData = imp6.getAbsoluteStructureData
    getConfiguredItemRevision = imp7.getConfiguredItemRevision
    getRevisionRules = imp2.getRevisionRules
    getVariantRules = imp2.getVariantRules
    queryClassicOptions = imp5.queryClassicOptions
    queryVariantConditions = imp5.queryVariantConditions
    reconfigureBOMWindows = imp6.reconfigureBOMWindows
    saveBOMWindows = imp6.saveBOMWindows
    startFindModelViews = imp3.startFindModelViews
    startReconcilePalette = imp3.startReconcilePalette
    writeAssemblyConfigurationDetails = imp21.writeAssemblyConfigurationDetails


class DataManagementService(TcService):
    createOrUpdateModelViewPalette = imp8.createOrUpdateModelViewPalette
    createOrUpdateModelViewProxies = imp8.createOrUpdateModelViewProxies
    createOrUpdateParts = imp9.createOrUpdateParts
    createOrUpdateParts2 = imp10.createOrUpdateParts
    createOrUpdateParts3 = imp11.createOrUpdateParts
    createOrUpdateParts4 = imp12.createOrUpdateParts
    createOrUpdateParts5 = imp13.createOrUpdateParts
    createOrUpdateParts6 = imp14.createOrUpdateParts
    createOrUpdateRelations = imp9.createOrUpdateRelations
    expandFoldersForCAD = imp9.expandFoldersForCAD
    expandFoldersForCAD2 = imp12.expandFoldersForCAD
    expandGRMRelations = imp9.expandGRMRelations
    expandPrimaryObjects = imp9.expandPrimaryObjects
    generateAlternateIds = imp9.generateAlternateIds
    getAllAttrMappings = imp9.getAllAttrMappings
    getAllAttrMappings2 = imp19.getAllAttrMappings2
    getAttrMappings = imp20.getAttrMappings
    getAttrMappingsForDatasetType = imp9.getAttrMappingsForDatasetType
    getAvailableTypes = imp9.getAvailableTypes
    resolveAttrMappings = imp11.resolveAttrMappings
    resolveAttrMappingsForDataset = imp9.resolveAttrMappingsForDataset


class AppSessionManagementService(TcService):
    createOrUpdateSavedSession = imp16.createOrUpdateSavedSession
    openSavedSession = imp16.openSavedSession



from __future__ import annotations

from tcsoa.gen.BusinessObjects import AppInterface
from tcsoa.gen.Ai._2010_09.Ai import GenerateAndEvaluateStructureResponse
from typing import List
from tcsoa.gen.Ai._2008_06.Ai import ObjectsWithConfig
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def generateAndEvaluateStructure(cls, aiObject: AppInterface, objectsToProcess: List[ObjectsWithConfig], exportTransferMode: str, absoluteXmlFileName: str, xpaths: List[str]) -> GenerateAndEvaluateStructureResponse:
        """
        Service to generate a plmxml based on the input objects, configuration, transfermode provided and  evaluate the
        specified xpaths 1.0 expressions against that plmxml. Optionally - a pre-existing xml file can be specified
        (via a full path accessible in tc server environment). In that case, only the xpaths argument is used along
        with the absoluteXmlFile argument.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='generateAndEvaluateStructure',
            library='Ai',
            service_date='2010_09',
            service_name='Ai',
            params={'aiObject': aiObject, 'objectsToProcess': objectsToProcess, 'exportTransferMode': exportTransferMode, 'absoluteXmlFileName': absoluteXmlFileName, 'xpaths': xpaths},
            response_cls=GenerateAndEvaluateStructureResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2019_06.ClosureRuleEditor import PrimaryToSecondaryTraversalMap, ClosureRuleTraversalInput, ClosureRuleTraversalResponse
from tcsoa.gen.Internal.Manufacturing._2019_06.ResourceManagement import GetICOMappingTargetsResponse
from typing import List
from tcsoa.gen.Internal.Manufacturing._2019_06.StructureSearch import ProximityCriteriaInput, PartsInProximityResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def searchPartsInProximityOfMFGFeatures(cls, criteriaInput: List[ProximityCriteriaInput]) -> PartsInProximityResponse:
        """
        This operation retrieves the BOMLine objects representing parts that are in proximity of a given BOMLine of
        type Mfg0BvrManufacturingFeature.
        
        Use cases:
        Manufacturing Planner wants to view the BOMLine objects that are in proximity of a given BOMLine of type
        Mfg0BvrManufacturingFeature to select which BOMLine objects to connect to the given BOMLine of type
        Mfg0BvrManufacturingFeature.
        """
        return cls.execute_soa_method(
            method_name='searchPartsInProximityOfMFGFeatures',
            library='Internal-Manufacturing',
            service_date='2019_06',
            service_name='StructureSearch',
            params={'criteriaInput': criteriaInput},
            response_cls=PartsInProximityResponse,
        )


class ClosureRuleEditorService(TcService):

    @classmethod
    def setClosureRuleTraversalInfo(cls, updatedTraversalInfo: PrimaryToSecondaryTraversalMap, closureRuleName: str) -> ServiceData:
        """
        This service operation updates the closure rule clauses as per the changes made in traversal of objects by the
        user.
        
        Use cases:
        Use Case 1: Updating the closure rule clauses in the closure rule editor view.
        This operation can be used to update the closure rule clauses for the selected closure rule based on the
        modifications done in objects traversal by the user.
        """
        return cls.execute_soa_method(
            method_name='setClosureRuleTraversalInfo',
            library='Internal-Manufacturing',
            service_date='2019_06',
            service_name='ClosureRuleEditor',
            params={'updatedTraversalInfo': updatedTraversalInfo, 'closureRuleName': closureRuleName},
            response_cls=ServiceData,
        )

    @classmethod
    def getClosureRuleTraversalInfo(cls, input: ClosureRuleTraversalInput) -> ClosureRuleTraversalResponse:
        """
        This service operation fetches the traversal information for the given closure rule and the top BOMLine object.
        
        Use cases:
        Use Case 1: User opens the closure rule editor window/panel.
        This operation can be used to get the traversal information for the selected closure rule and BOMLine object
        and display it in the closure rule editor window/panel.
        """
        return cls.execute_soa_method(
            method_name='getClosureRuleTraversalInfo',
            library='Internal-Manufacturing',
            service_date='2019_06',
            service_name='ClosureRuleEditor',
            params={'input': input},
            response_cls=ClosureRuleTraversalResponse,
        )


class ResourceManagementService(TcService):

    @classmethod
    def getICOMappingTargets(cls, icoUIDs: List[str], viewType: int) -> GetICOMappingTargetsResponse:
        """
        This operation obtains target classes under MRL and views that a given source classification object (ICO) can
        be mapped to. Target classes are obtained using following steps. If a match is found at any step, the algorithm
        returns target classes at this step and next steps are not executed.
        1.    Search for the direct mapping classes based on class ID.
        2.    Search is done based on connection code attribute values of source ICO.
        3.    Vendor specific prefixes from the class associated with the ICO are removed and search is done.
        4.    "Bbased on" information in the source class is read and search is done.
        
        Use cases:
        The API will internally extract value for the class ID associated with input source ICO. 
        Checks will be done in following order.
        1.    Search for all views whose IDs match the given source class ID. If source class ID is
        "IS#MILTHGI_WIS$$MTH_MZYL10", search directly for "IS#MILTHGI_WIS$$MTH_MZYL10" views.
        2.    Read values of connection code attributes for source ICO. Obtain strings for 
        a.    GTC base class: <GTC_BASE>
        b.    The connection to the machine side: <MACH>
        c.    The connection to the workpiece side: <WKPS>
        Find mapping targets for above combination.
        3.    If mapping targets are not found, use source class ID to obtain strings for <GTC_BASE>, <MACH> and
        <WKPS>. Find mapping targets for this combination.
        """
        return cls.execute_soa_method(
            method_name='getICOMappingTargets',
            library='Internal-Manufacturing',
            service_date='2019_06',
            service_name='ResourceManagement',
            params={'icoUIDs': icoUIDs, 'viewType': viewType},
            response_cls=GetICOMappingTargetsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.BusinessModeler._2010_04.DataModelManagement import DataModelDeploymentInput
from tcsoa.gen.Internal.BusinessModeler._2010_09.DataModelManagement import DeployDataModelResponse, SiteTemplateDeployInfoResponse
from typing import List
from tcsoa.utils import TcService


class DataModelManagementService(TcService):

    @classmethod
    def getSiteTemplateDeployInfo(cls, templateNames: List[str]) -> SiteTemplateDeployInfoResponse:
        """
        This operation gets the site information from the current database for the requested list of templates
        ('templateNames').
        
        Exceptions:
        >ServiceException    If the template is not present in the current database (216016).
        """
        return cls.execute_soa_method(
            method_name='getSiteTemplateDeployInfo',
            library='Internal-BusinessModeler',
            service_date='2010_09',
            service_name='DataModelManagement',
            params={'templateNames': templateNames},
            response_cls=SiteTemplateDeployInfoResponse,
        )

    @classmethod
    def deployDataModel2(cls, deployOption: str, updaterUpdateOption: str, updaterModeOption: str, inputs: List[DataModelDeploymentInput]) -> DeployDataModelResponse:
        """
        The user invoking this service should have the system administrative privileges. This operation updates the
        database with the data model changes available in the template project referred in the 'inputs' argument. Based
        on the 'deployOtion' argument value the template will be live deployed or updated. The service acquires a
        database lock before updating the data model so there should not be any active lock before running this service.
        
        Validations done before updating the data model
        - Validate if database is free of any lock.
        - Validate the privileges of the user invoking the service.
        - If preference 'BMIDE_ALLOW_FULL_DEPLOY_FROM_CLIENT' is set to 'FALSE' or 'BMIDE_ALLOW_OPS_DEPLOY' is set to
        'FALSE' in the server then live deploy or live update is restricted.
        - The template that needs to be updated or deployed should have all its dependencies deployed.
        - If the template is being live updated then elements present in the delta should be enabled for the live
        update operation.
        - The element present in the delta files is eligible for live deploy if not then proper error message is
        returned.
        
        """
        return cls.execute_soa_method(
            method_name='deployDataModel2',
            library='Internal-BusinessModeler',
            service_date='2010_09',
            service_name='DataModelManagement',
            params={'deployOption': deployOption, 'updaterUpdateOption': updaterUpdateOption, 'updaterModeOption': updaterModeOption, 'inputs': inputs},
            response_cls=DeployDataModelResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, EPMAssignmentList, EPMSignoffProfile
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class TaskAssignmentsInput(TcBaseObj):
    """
    This is the input structure for getWorkflowTaskAssignments SOA.
    
    :var taskOrTemplate: EPMTask or the EPMTaskTemplate for which the assignments data is required.
    :var pal: The EPMAssignmentList from which the data needs to be fetched. Either the taskOrTemplate or this
    parameter would exist.
    :var operationMode: Criteria on which the tasks for assignments are filtered. Value 1 indicates all tasks and 2 for
    future tasks.
    :var startIndex: Index of the start element of the response data. If this is not given, it will be considered as 0.
    :var maxToLoad: Maximum elements to load and be returned in the response data. If negative, all of the found
    objects are loaded.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var additionalData: A map (string, list of string) of filter and sort criteria. The keys for this map are:
     "sort_criteria"  = "task_name", "origin" or "assignee". 
     "filter_criteria" = "task_name", "origin" or "assignee".
    """
    taskOrTemplate: BusinessObject = None
    pal: EPMAssignmentList = None
    operationMode: int = 0
    startIndex: int = 0
    maxToLoad: int = 0
    clientId: str = ''
    additionalData: KeyValuePair = None


@dataclass
class TaskAssignmentsOutput(TcBaseObj):
    """
    This structure represents EPMTask assignment information associated with the EPMJob.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with input structure.
    :var outData: A list of task data that contains information about the task assignments, assignment types and their
    origin.
    :var dpData: A list of dynamic Participant objects . The key is a specific dynamic Participant type.
    :var additionalData: A map (string, list of strings) of dynamic Participant type name as key and the value is the
    list of tasks (EPMTask) or (Signoff) that refers it.
    """
    clientId: str = ''
    outData: List[TaskData] = ()
    dpData: DynamicParticipantsPair = None
    additionalData: KeyValuePair = None


@dataclass
class TaskAssignmentsResponse(TcBaseObj):
    """
    This structure represents response for getWorkflowTaskAssignments SOA.
    
    :var output: A list of TaskAssignmentsOutput structure.
    :var serviceData: The service data.
    """
    output: List[TaskAssignmentsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class TaskData(TcBaseObj):
    """
    This structure contains all the assignment information related to task or task template.
    
    :var task: The task (EPMTask) or task template (EPMTaskTemplate)  for which the assigments are to be updated.
    :var assigmentData: A map (string, list of AssignmentData)  of task assignment type string as the key. The types of
    task assigments include "Reviewers", "Acknowledgers", "Notifiers" and "Assignee". The value is list of
    AssignmentData.
    :var additionalData: A map (string, list of strings) that has sources from which the assignees are referred by the
    EPMTask. For example, dynamic Participants. The key to this map is same like the assignmentData map above. The
    value is the list of origins. For example, if the Reviewer of the EPMTask is from Participant types:
    "ProposedReviewes" and "ChangeSpecialist", then the key to this map is "Reviewers" and it points to a list
    containing two values: "ProposedReviewers" and "ChangeSpecialist". 
    
    "Reviewers" = list of participant types
    "Acknowledgers" = list of participant types
    "Assignee" = participant type
    "Notifiers" = list of participant types
    """
    task: BusinessObject = None
    assigmentData: TaskAssignmentsMap = None
    additionalData: KeyValuePair = None


@dataclass
class UpdateTaskAssignmentsInput(TcBaseObj):
    """
    This structure contains information about the task and dynamic participants assignments.
    
    Note: Values for either taskData or dpData will be present in this structure at a time. For task data updates, this
    will have taskData details present and dpData would be NULL. For updating the dpData details, the taskData values
    will point to NULL.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var taskData: A task and the assignees that needs to be updated. It can also contain Signoff data. The assignment
    data for each task is grouped as "Reviewers", "Assignee", "Notifiers" and "Acknowledgers".
    :var dpData: A Participant data that contains details about the type and the new list of assignees to be updated.
    :var additionalData: A map (string, list of strings) of details about task assignments. The keys and the values are
    :
    " task_bypass"= "yes" or "no"
    " late_task" = "yes" or "no"
    " task_pal_resource" = list indicating the name of EPMAssignmentList objects.
    """
    clientId: str = ''
    taskData: TaskData = None
    dpData: DynamicParticipantsData = None
    additionalData: KeyValuePair = None


@dataclass
class UpdateTaskAssignmentsOutput(TcBaseObj):
    """
    This structure contains information about the successfully updated objects.
    
    :var clientId: The clientID received to uniquely identify this task.
    :var taskOrDP: The EPMTask or Participant that has been successfully updated.
    """
    clientId: str = ''
    taskOrDP: BusinessObject = None


@dataclass
class UpdateTaskAssignmentsReponse(TcBaseObj):
    """
    This structure contains the list of sucessfully updated objects.
    
    :var outData: A list of EPMTask  or Participant objects.
    :var serviceData: The service data that may also contain errors.
    """
    outData: List[UpdateTaskAssignmentsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class AssignmentData(TcBaseObj):
    """
    This represent all the assignment information like origin, signoff profile and decision is required or not.
    
    :var member: The task assignment. Supported types are: User, GroupMember or Resourcepool.
    :var origin: The origin source of the member. For example EPMAssignmentList, Participant Type, TcProject,
    Resourcepool and ImanAliasList .
    :var signoffProfile: The signoff profile if the member is part of one.
    :var isRequired: If true, the decision on the Signoff is mandatory, otherwise not.
    :var additionalData: A map (string, list of strings) that has details related to EPMTask or Signoff. 
        "task_comments" = User added comments for task.
        "signoff_comments" = User added comments for signoff.
    """
    member: BusinessObject = None
    origin: BusinessObject = None
    signoffProfile: EPMSignoffProfile = None
    isRequired: bool = False
    additionalData: KeyValuePair = None


@dataclass
class DynamicParticipantsData(TcBaseObj):
    """
    This structure represent all the information related to particular Participant Type which is being used in task or
    process template.
    
    :var primaryObject: A source object to which Participant is attached to. It is WorkspaceObject either a workflow
    target or process (EPMJob) itself.
    :var internalName: The internal name of the Participant type.
    :var displayName: The display name of the Participant type.
    :var allowMultipleAssignee: Identifies if the Participant type supports multiple assignees or it is a single
    assignee. If true, multiple assignees can be added otherwise this it supports single assignee.
    :var assigneeList: A list of all assignees for this Participant type that are applicable to the workflow.
    :var additionalData: A map (string, list of strings) of eligibility criteria name for the Participant and the
    supported list of values . The supported key and values are:
     "participant_used_on_obj_type" = list of WorkspaceObjects that this Participant type supports.
     "participant_eligibility" = list of group::role criteria for assigning Participant type.
    """
    primaryObject: BusinessObject = None
    internalName: str = ''
    displayName: str = ''
    allowMultipleAssignee: bool = False
    assigneeList: List[BusinessObject] = ()
    additionalData: KeyValuePair = None


"""
This map will be used to pass additonal information.
"""
KeyValuePair = Dict[str, List[str]]


"""
This map will store task assignment information such as "Assignee", "Reviewers", "Acknowledgers" and "Notifiers" etc.
"""
TaskAssignmentsMap = Dict[str, List[AssignmentData]]


"""
This map will contain key as Partcipant Type name and value will DynamicParticipantsData structure which have all information related to that Participant Type.
"""
DynamicParticipantsPair = Dict[str, DynamicParticipantsData]



from __future__ import annotations

from tcsoa.gen.UiConfig._2014_11.UiConfig import GetUIConfigResponse, SaveUiConfigurations, GetUIConfigInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class UiConfigService(TcService):

    @classmethod
    def getUIConfigs(cls, getUiConfigsIn: List[GetUIConfigInput]) -> GetUIConfigResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes command and column configuration information.
        
        Use cases:
        Use Case 1: Request UI Configuration(s) based on the current login user
        Client requests the column and/or command information for one or more client scopes using this operation and
        scope as login user.
        
        Use Case 2: Request UI Configuration(s) based on a specific Teamcenter scope
        Client requests the column and/or command information for one or more client scopes using this operation and
        scope as Role, Site or Group.
        """
        return cls.execute_soa_method(
            method_name='getUIConfigs',
            library='UiConfig',
            service_date='2014_11',
            service_name='UiConfig',
            params={'getUiConfigsIn': getUiConfigsIn},
            response_cls=GetUIConfigResponse,
        )

    @classmethod
    def saveUiConfigs(cls, saveUiConfigsIn: SaveUiConfigurations) -> ServiceData:
        """
        This service operation saves column and command configuration information to the Teamcenter database.  A
        Teamcenter client may use this information to render the user interface as the user navigates the Teamcenter
        client.  Default configurations are created using the Business Modeler IDE and installed with Teamcenter.
        
        Use cases:
        Use Case 1: User level column configuration
        
        A User wants to change the default column configuration for a Client Scope when he is logged into the client. 
        The saved configuration will override the default configuration for the user's applicable role, group or site.  
        
        Use Case 2: Administrator level column and command configuration
        An Administrator wants to change the UI configurations for Client Scope URI for a specific Group, Role, or
        User.  Columns may be reordered, removed, or if removed, added back to a specific configuration.  Commands and
        command collections may be reordered, hidden or unhidden.  The Business Modeler IDE must be used to create new
        commands, commands and command collections.
        """
        return cls.execute_soa_method(
            method_name='saveUiConfigs',
            library='UiConfig',
            service_date='2014_11',
            service_name='UiConfig',
            params={'saveUiConfigsIn': saveUiConfigsIn},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class UnreadMessages(TcBaseObj):
    """
    A list of Fnd0Message UID for the unread messages.
    
    :var messages: The list of Fnd0Message UIDs.
    """
    messages: List[str] = ()



from __future__ import annotations

from tcsoa.gen.EditContext._2016_10.DataManagement import ChangeContextMapIn, SetChangeContextResponse2
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def setChangeContext2(cls, inputs: ChangeContextMapIn) -> SetChangeContextResponse2:
        """
        This operation  configures business objects to allow the objects to be edited, queried or deleted in a context
        without changing the public versions of the objects. This operation can also configure business objects with no
        context in order to edit, query or delete the public versions of the objects.
        This operation does not have any effect on business objects that are not minor revisable and such objects will
        always be returned without being configured.
        The operation can be used to configure multiple sets of objects using different contexts. 
        
        Use Case:
        Use Case 1: User wants to edit the properties of a set of business objects in an isolated context with the
        intention to share the proposed changes with other users.
        
        - User creates a Fnd0MarkupSpace object that abstracts the context within which the edit needs to be performed.
        - The Fnd0MarkupSpace object is provided as input to the operation along with the business objects that need to
        be edited.
        - The configured business objects are returned in the response and can be used for performing edits in the
        context.
        
        
        
        Use Case 2: User wants to query the properties of a set of business objects in an isolated context that have
        been edited in the context of a ChangeNotice.
        
        - User creates a ChangeNotice object that abstracts the context within which the edit needs to be performed.
        - The ChangeNotice object is provided as input to the operation along with the business objects that need to be
        configured.
        - The configured business objects are returned in the response and can be used for performing edits in the
        context.
        
        
        
        
        Use Case 3: User wants to query a property belonging to the public version of a business object that is
        currently configured for editing in a collaborative space.
        
        - User passes a null value for the context along with the business object that needs to be inspected in public.
        - The unconfigured business objects are returned in the response and any queries performed on the objects will
        return public data.
        
        """
        return cls.execute_soa_method(
            method_name='setChangeContext2',
            library='EditContext',
            service_date='2016_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SetChangeContextResponse2,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.ProjectManagementAw._2016_12.ScheduleManagementAw import ReplaceMemberAssignmentData, LoadScheduleResponse, LoadScheduleInfo
from tcsoa.gen.BusinessObjects import ScheduleMember
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def loadSchedule(cls, loadScheduleInfo: LoadScheduleInfo) -> LoadScheduleResponse:
        """
        Loads the ScheduleTask objects of a given Schedule based on the reference ScheduleTask. The number of
        ScheduleTask objects to be loaded will be determined by AWC_DefaultPageSize preference or &lsquo;50&rsquo;, if
        the preference is not found. This operation also loads all the Fnd0ProxyTask objects and TaskDependency
        relations which are referencing the loaded ScheduleTask objects.
        """
        return cls.execute_soa_method(
            method_name='loadSchedule',
            library='ProjectManagementAw',
            service_date='2016_12',
            service_name='ScheduleManagementAw',
            params={'loadScheduleInfo': loadScheduleInfo},
            response_cls=LoadScheduleResponse,
        )

    @classmethod
    def removeAssignments(cls, membersToRemoveAssignments: List[ScheduleMember], runInBackground: bool) -> ServiceData:
        """
        This operation removes all assignments of a ScheduleMember. The system will not delete the ScheduleMember
        object.
        """
        return cls.execute_soa_method(
            method_name='removeAssignments',
            library='ProjectManagementAw',
            service_date='2016_12',
            service_name='ScheduleManagementAw',
            params={'membersToRemoveAssignments': membersToRemoveAssignments, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def removeAssignmentsAsync(cls, membersToRemoveAssignments: List[ScheduleMember]) -> None:
        """
        This operation removes all assignments of a ScheduleMember specified in the input. This operation runs
        asynchronously in its own server in the background.
        """
        return cls.execute_soa_method(
            method_name='removeAssignmentsAsync',
            library='ProjectManagementAw',
            service_date='2016_12',
            service_name='ScheduleManagementAw',
            params={'membersToRemoveAssignments': membersToRemoveAssignments},
            response_cls=None,
        )

    @classmethod
    def replaceMemberAssignments(cls, replaceMemberAssignmentData: List[ReplaceMemberAssignmentData], runInBackground: bool) -> ServiceData:
        """
        This operation removes all assignments of a ScheduleMember and assigns them to a new ScheduleMember. While
        replacing the assignments, the system will check if the new resource is a ScheduleMember. If not, the system
        will create a ScheduleMember for the resource and replace the assignments.
        """
        return cls.execute_soa_method(
            method_name='replaceMemberAssignments',
            library='ProjectManagementAw',
            service_date='2016_12',
            service_name='ScheduleManagementAw',
            params={'replaceMemberAssignmentData': replaceMemberAssignmentData, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def replaceMemberAssignmentsAsync(cls, replaceMemberAssignmentData: List[ReplaceMemberAssignmentData]) -> None:
        """
        This operation removes all assignments of a ScheduleMember and assigns them to a new ScheduleMember. While
        replacing the assignments, the system will check if the new resource is a ScheduleMember. If not, the system
        will create a ScheduleMember for the resource and replace the assignments. This operation runs asynchronously
        in its own server in the background.
        """
        return cls.execute_soa_method(
            method_name='replaceMemberAssignmentsAsync',
            library='ProjectManagementAw',
            service_date='2016_12',
            service_name='ScheduleManagementAw',
            params={'replaceMemberAssignmentData': replaceMemberAssignmentData},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.StructureManagement._2017_05.StructureSearch import SearchScope, StructureSearchResultResponse, SearchExpressionSet
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def nextSearch2(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        This operation gets the next set of search results, which was initialized by the startSearch2 operation. This
        operation returns the results in batches and needs to be called repeatedly until the flag for search complete
        is true in the response. Input to this operation is the search cursor object which was returned by the
        startSearch2 or a previous call to nextSearch2 operation.
        
        Use cases:
        A user wants to perform structure search within a particular scope. The user needs to select search
        criteria(s), from the supported list to create search expression and start search operation. Once the initial
        set of a search result is returned, this operation will be used to get the next set of search result.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='nextSearch2',
            library='StructureManagement',
            service_date='2017_05',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def startSearch2(cls, scope: SearchScope, searchExpression: SearchExpressionSet, returnLiteLines: bool) -> StructureSearchResultResponse:
        """
        This operation initializes the structure search. The input to the operation is a search expression set and the
        scope to perform the search in.
        
        Use cases:
        A user wants to perform structure search within a particular scope. The user needs to select search criteria(s)
        from the supported list to create search expression and start search operation.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='startSearch2',
            library='StructureManagement',
            service_date='2017_05',
            service_name='StructureSearch',
            params={'scope': scope, 'searchExpression': searchExpression, 'returnLiteLines': returnLiteLines},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def stopSearch2(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        This operation stops the current search identified by the search cursor object. The input to the operation is
        the search cursor object which was returned by the startSearch2 or previous call to nextSearch2 operation.
        
        Use cases:
        A user wants to perform Cacheless search within a particular scope. The user needs to select search criteria(s)
        from the supported list to create search expression and start search operation. Once the initial batch of
        search result is returned, this operation can be used to stop the search.
        """
        return cls.execute_soa_method(
            method_name='stopSearch2',
            library='StructureManagement',
            service_date='2017_05',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )
